import React, { useRef, useState, useEffect, createContext, useMemo, useContext, useCallback, useImperativeHandle, useLayoutEffect, createRef } from 'react';
import { parseMetadata, suffixMetadata, noop } from '@highlight-ui/utils-commons';
import { useForkRef, useAutoPosition, useClickOutside, useId } from '@highlight-ui/utils-hooks';
import classnames from 'classnames';
import { Icon } from '@highlight-ui/icon';
import { Checkbox } from '@highlight-ui/checkbox';
import { Avatar } from '@highlight-ui/avatar';
import { Typography } from '@highlight-ui/typography';
import { TextInput, InputContainer } from '@highlight-ui/input';
import { Button, IconButton } from '@highlight-ui/button';
import { ColorPicker } from '@highlight-ui/color-picker';

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function () {
  __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
  };
  return __assign.apply(this, arguments);
};
function __rest(s, e) {
  var t = {};
  for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0) t[p] = s[p];
  if (s != null && typeof Object.getOwnPropertySymbols === "function") for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
    if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i])) t[p[i]] = s[p[i]];
  }
  return t;
}
function __spreadArray(to, from, pack) {
  if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
    if (ar || !(i in from)) {
      if (!ar) ar = Array.prototype.slice.call(from, 0, i);
      ar[i] = from[i];
    }
  }
  return to.concat(ar || Array.prototype.slice.call(from));
}
var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};
var lodash_groupby = {
  exports: {}
};

/**
 * lodash (Custom Build) <https://lodash.com/>
 * Build: `lodash modularize exports="npm" -o ./`
 * Copyright jQuery Foundation and other contributors <https://jquery.org/>
 * Released under MIT license <https://lodash.com/license>
 * Based on Underscore.js 1.8.3 <http://underscorejs.org/LICENSE>
 * Copyright Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
 */

(function (module, exports) {
  /** Used as the size to enable large array optimizations. */
  var LARGE_ARRAY_SIZE = 200;

  /** Used as the `TypeError` message for "Functions" methods. */
  var FUNC_ERROR_TEXT = 'Expected a function';

  /** Used to stand-in for `undefined` hash values. */
  var HASH_UNDEFINED = '__lodash_hash_undefined__';

  /** Used to compose bitmasks for comparison styles. */
  var UNORDERED_COMPARE_FLAG = 1,
    PARTIAL_COMPARE_FLAG = 2;

  /** Used as references for various `Number` constants. */
  var INFINITY = 1 / 0,
    MAX_SAFE_INTEGER = 9007199254740991;

  /** `Object#toString` result references. */
  var argsTag = '[object Arguments]',
    arrayTag = '[object Array]',
    boolTag = '[object Boolean]',
    dateTag = '[object Date]',
    errorTag = '[object Error]',
    funcTag = '[object Function]',
    genTag = '[object GeneratorFunction]',
    mapTag = '[object Map]',
    numberTag = '[object Number]',
    objectTag = '[object Object]',
    promiseTag = '[object Promise]',
    regexpTag = '[object RegExp]',
    setTag = '[object Set]',
    stringTag = '[object String]',
    symbolTag = '[object Symbol]',
    weakMapTag = '[object WeakMap]';
  var arrayBufferTag = '[object ArrayBuffer]',
    dataViewTag = '[object DataView]',
    float32Tag = '[object Float32Array]',
    float64Tag = '[object Float64Array]',
    int8Tag = '[object Int8Array]',
    int16Tag = '[object Int16Array]',
    int32Tag = '[object Int32Array]',
    uint8Tag = '[object Uint8Array]',
    uint8ClampedTag = '[object Uint8ClampedArray]',
    uint16Tag = '[object Uint16Array]',
    uint32Tag = '[object Uint32Array]';

  /** Used to match property names within property paths. */
  var reIsDeepProp = /\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,
    reIsPlainProp = /^\w*$/,
    reLeadingDot = /^\./,
    rePropName = /[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g;

  /**
   * Used to match `RegExp`
   * [syntax characters](http://ecma-international.org/ecma-262/7.0/#sec-patterns).
   */
  var reRegExpChar = /[\\^$.*+?()[\]{}|]/g;

  /** Used to match backslashes in property paths. */
  var reEscapeChar = /\\(\\)?/g;

  /** Used to detect host constructors (Safari). */
  var reIsHostCtor = /^\[object .+?Constructor\]$/;

  /** Used to detect unsigned integer values. */
  var reIsUint = /^(?:0|[1-9]\d*)$/;

  /** Used to identify `toStringTag` values of typed arrays. */
  var typedArrayTags = {};
  typedArrayTags[float32Tag] = typedArrayTags[float64Tag] = typedArrayTags[int8Tag] = typedArrayTags[int16Tag] = typedArrayTags[int32Tag] = typedArrayTags[uint8Tag] = typedArrayTags[uint8ClampedTag] = typedArrayTags[uint16Tag] = typedArrayTags[uint32Tag] = true;
  typedArrayTags[argsTag] = typedArrayTags[arrayTag] = typedArrayTags[arrayBufferTag] = typedArrayTags[boolTag] = typedArrayTags[dataViewTag] = typedArrayTags[dateTag] = typedArrayTags[errorTag] = typedArrayTags[funcTag] = typedArrayTags[mapTag] = typedArrayTags[numberTag] = typedArrayTags[objectTag] = typedArrayTags[regexpTag] = typedArrayTags[setTag] = typedArrayTags[stringTag] = typedArrayTags[weakMapTag] = false;

  /** Detect free variable `global` from Node.js. */
  var freeGlobal = typeof commonjsGlobal == 'object' && commonjsGlobal && commonjsGlobal.Object === Object && commonjsGlobal;

  /** Detect free variable `self`. */
  var freeSelf = typeof self == 'object' && self && self.Object === Object && self;

  /** Used as a reference to the global object. */
  var root = freeGlobal || freeSelf || Function('return this')();

  /** Detect free variable `exports`. */
  var freeExports = exports && !exports.nodeType && exports;

  /** Detect free variable `module`. */
  var freeModule = freeExports && 'object' == 'object' && module && !module.nodeType && module;

  /** Detect the popular CommonJS extension `module.exports`. */
  var moduleExports = freeModule && freeModule.exports === freeExports;

  /** Detect free variable `process` from Node.js. */
  var freeProcess = moduleExports && freeGlobal.process;

  /** Used to access faster Node.js helpers. */
  var nodeUtil = function () {
    try {
      return freeProcess && freeProcess.binding('util');
    } catch (e) {}
  }();

  /* Node.js helper references. */
  var nodeIsTypedArray = nodeUtil && nodeUtil.isTypedArray;

  /**
   * A specialized version of `baseAggregator` for arrays.
   *
   * @private
   * @param {Array} [array] The array to iterate over.
   * @param {Function} setter The function to set `accumulator` values.
   * @param {Function} iteratee The iteratee to transform keys.
   * @param {Object} accumulator The initial aggregated object.
   * @returns {Function} Returns `accumulator`.
   */
  function arrayAggregator(array, setter, iteratee, accumulator) {
    var index = -1,
      length = array ? array.length : 0;
    while (++index < length) {
      var value = array[index];
      setter(accumulator, value, iteratee(value), array);
    }
    return accumulator;
  }

  /**
   * A specialized version of `_.some` for arrays without support for iteratee
   * shorthands.
   *
   * @private
   * @param {Array} [array] The array to iterate over.
   * @param {Function} predicate The function invoked per iteration.
   * @returns {boolean} Returns `true` if any element passes the predicate check,
   *  else `false`.
   */
  function arraySome(array, predicate) {
    var index = -1,
      length = array ? array.length : 0;
    while (++index < length) {
      if (predicate(array[index], index, array)) {
        return true;
      }
    }
    return false;
  }

  /**
   * The base implementation of `_.property` without support for deep paths.
   *
   * @private
   * @param {string} key The key of the property to get.
   * @returns {Function} Returns the new accessor function.
   */
  function baseProperty(key) {
    return function (object) {
      return object == null ? undefined : object[key];
    };
  }

  /**
   * The base implementation of `_.times` without support for iteratee shorthands
   * or max array length checks.
   *
   * @private
   * @param {number} n The number of times to invoke `iteratee`.
   * @param {Function} iteratee The function invoked per iteration.
   * @returns {Array} Returns the array of results.
   */
  function baseTimes(n, iteratee) {
    var index = -1,
      result = Array(n);
    while (++index < n) {
      result[index] = iteratee(index);
    }
    return result;
  }

  /**
   * The base implementation of `_.unary` without support for storing metadata.
   *
   * @private
   * @param {Function} func The function to cap arguments for.
   * @returns {Function} Returns the new capped function.
   */
  function baseUnary(func) {
    return function (value) {
      return func(value);
    };
  }

  /**
   * Gets the value at `key` of `object`.
   *
   * @private
   * @param {Object} [object] The object to query.
   * @param {string} key The key of the property to get.
   * @returns {*} Returns the property value.
   */
  function getValue(object, key) {
    return object == null ? undefined : object[key];
  }

  /**
   * Checks if `value` is a host object in IE < 9.
   *
   * @private
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is a host object, else `false`.
   */
  function isHostObject(value) {
    // Many host objects are `Object` objects that can coerce to strings
    // despite having improperly defined `toString` methods.
    var result = false;
    if (value != null && typeof value.toString != 'function') {
      try {
        result = !!(value + '');
      } catch (e) {}
    }
    return result;
  }

  /**
   * Converts `map` to its key-value pairs.
   *
   * @private
   * @param {Object} map The map to convert.
   * @returns {Array} Returns the key-value pairs.
   */
  function mapToArray(map) {
    var index = -1,
      result = Array(map.size);
    map.forEach(function (value, key) {
      result[++index] = [key, value];
    });
    return result;
  }

  /**
   * Creates a unary function that invokes `func` with its argument transformed.
   *
   * @private
   * @param {Function} func The function to wrap.
   * @param {Function} transform The argument transform.
   * @returns {Function} Returns the new function.
   */
  function overArg(func, transform) {
    return function (arg) {
      return func(transform(arg));
    };
  }

  /**
   * Converts `set` to an array of its values.
   *
   * @private
   * @param {Object} set The set to convert.
   * @returns {Array} Returns the values.
   */
  function setToArray(set) {
    var index = -1,
      result = Array(set.size);
    set.forEach(function (value) {
      result[++index] = value;
    });
    return result;
  }

  /** Used for built-in method references. */
  var arrayProto = Array.prototype,
    funcProto = Function.prototype,
    objectProto = Object.prototype;

  /** Used to detect overreaching core-js shims. */
  var coreJsData = root['__core-js_shared__'];

  /** Used to detect methods masquerading as native. */
  var maskSrcKey = function () {
    var uid = /[^.]+$/.exec(coreJsData && coreJsData.keys && coreJsData.keys.IE_PROTO || '');
    return uid ? 'Symbol(src)_1.' + uid : '';
  }();

  /** Used to resolve the decompiled source of functions. */
  var funcToString = funcProto.toString;

  /** Used to check objects for own properties. */
  var hasOwnProperty = objectProto.hasOwnProperty;

  /**
   * Used to resolve the
   * [`toStringTag`](http://ecma-international.org/ecma-262/7.0/#sec-object.prototype.tostring)
   * of values.
   */
  var objectToString = objectProto.toString;

  /** Used to detect if a method is native. */
  var reIsNative = RegExp('^' + funcToString.call(hasOwnProperty).replace(reRegExpChar, '\\$&').replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g, '$1.*?') + '$');

  /** Built-in value references. */
  var Symbol = root.Symbol,
    Uint8Array = root.Uint8Array,
    propertyIsEnumerable = objectProto.propertyIsEnumerable,
    splice = arrayProto.splice;

  /* Built-in method references for those with the same name as other `lodash` methods. */
  var nativeKeys = overArg(Object.keys, Object);

  /* Built-in method references that are verified to be native. */
  var DataView = getNative(root, 'DataView'),
    Map = getNative(root, 'Map'),
    Promise = getNative(root, 'Promise'),
    Set = getNative(root, 'Set'),
    WeakMap = getNative(root, 'WeakMap'),
    nativeCreate = getNative(Object, 'create');

  /** Used to detect maps, sets, and weakmaps. */
  var dataViewCtorString = toSource(DataView),
    mapCtorString = toSource(Map),
    promiseCtorString = toSource(Promise),
    setCtorString = toSource(Set),
    weakMapCtorString = toSource(WeakMap);

  /** Used to convert symbols to primitives and strings. */
  var symbolProto = Symbol ? Symbol.prototype : undefined,
    symbolValueOf = symbolProto ? symbolProto.valueOf : undefined,
    symbolToString = symbolProto ? symbolProto.toString : undefined;

  /**
   * Creates a hash object.
   *
   * @private
   * @constructor
   * @param {Array} [entries] The key-value pairs to cache.
   */
  function Hash(entries) {
    var index = -1,
      length = entries ? entries.length : 0;
    this.clear();
    while (++index < length) {
      var entry = entries[index];
      this.set(entry[0], entry[1]);
    }
  }

  /**
   * Removes all key-value entries from the hash.
   *
   * @private
   * @name clear
   * @memberOf Hash
   */
  function hashClear() {
    this.__data__ = nativeCreate ? nativeCreate(null) : {};
  }

  /**
   * Removes `key` and its value from the hash.
   *
   * @private
   * @name delete
   * @memberOf Hash
   * @param {Object} hash The hash to modify.
   * @param {string} key The key of the value to remove.
   * @returns {boolean} Returns `true` if the entry was removed, else `false`.
   */
  function hashDelete(key) {
    return this.has(key) && delete this.__data__[key];
  }

  /**
   * Gets the hash value for `key`.
   *
   * @private
   * @name get
   * @memberOf Hash
   * @param {string} key The key of the value to get.
   * @returns {*} Returns the entry value.
   */
  function hashGet(key) {
    var data = this.__data__;
    if (nativeCreate) {
      var result = data[key];
      return result === HASH_UNDEFINED ? undefined : result;
    }
    return hasOwnProperty.call(data, key) ? data[key] : undefined;
  }

  /**
   * Checks if a hash value for `key` exists.
   *
   * @private
   * @name has
   * @memberOf Hash
   * @param {string} key The key of the entry to check.
   * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
   */
  function hashHas(key) {
    var data = this.__data__;
    return nativeCreate ? data[key] !== undefined : hasOwnProperty.call(data, key);
  }

  /**
   * Sets the hash `key` to `value`.
   *
   * @private
   * @name set
   * @memberOf Hash
   * @param {string} key The key of the value to set.
   * @param {*} value The value to set.
   * @returns {Object} Returns the hash instance.
   */
  function hashSet(key, value) {
    var data = this.__data__;
    data[key] = nativeCreate && value === undefined ? HASH_UNDEFINED : value;
    return this;
  }

  // Add methods to `Hash`.
  Hash.prototype.clear = hashClear;
  Hash.prototype['delete'] = hashDelete;
  Hash.prototype.get = hashGet;
  Hash.prototype.has = hashHas;
  Hash.prototype.set = hashSet;

  /**
   * Creates an list cache object.
   *
   * @private
   * @constructor
   * @param {Array} [entries] The key-value pairs to cache.
   */
  function ListCache(entries) {
    var index = -1,
      length = entries ? entries.length : 0;
    this.clear();
    while (++index < length) {
      var entry = entries[index];
      this.set(entry[0], entry[1]);
    }
  }

  /**
   * Removes all key-value entries from the list cache.
   *
   * @private
   * @name clear
   * @memberOf ListCache
   */
  function listCacheClear() {
    this.__data__ = [];
  }

  /**
   * Removes `key` and its value from the list cache.
   *
   * @private
   * @name delete
   * @memberOf ListCache
   * @param {string} key The key of the value to remove.
   * @returns {boolean} Returns `true` if the entry was removed, else `false`.
   */
  function listCacheDelete(key) {
    var data = this.__data__,
      index = assocIndexOf(data, key);
    if (index < 0) {
      return false;
    }
    var lastIndex = data.length - 1;
    if (index == lastIndex) {
      data.pop();
    } else {
      splice.call(data, index, 1);
    }
    return true;
  }

  /**
   * Gets the list cache value for `key`.
   *
   * @private
   * @name get
   * @memberOf ListCache
   * @param {string} key The key of the value to get.
   * @returns {*} Returns the entry value.
   */
  function listCacheGet(key) {
    var data = this.__data__,
      index = assocIndexOf(data, key);
    return index < 0 ? undefined : data[index][1];
  }

  /**
   * Checks if a list cache value for `key` exists.
   *
   * @private
   * @name has
   * @memberOf ListCache
   * @param {string} key The key of the entry to check.
   * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
   */
  function listCacheHas(key) {
    return assocIndexOf(this.__data__, key) > -1;
  }

  /**
   * Sets the list cache `key` to `value`.
   *
   * @private
   * @name set
   * @memberOf ListCache
   * @param {string} key The key of the value to set.
   * @param {*} value The value to set.
   * @returns {Object} Returns the list cache instance.
   */
  function listCacheSet(key, value) {
    var data = this.__data__,
      index = assocIndexOf(data, key);
    if (index < 0) {
      data.push([key, value]);
    } else {
      data[index][1] = value;
    }
    return this;
  }

  // Add methods to `ListCache`.
  ListCache.prototype.clear = listCacheClear;
  ListCache.prototype['delete'] = listCacheDelete;
  ListCache.prototype.get = listCacheGet;
  ListCache.prototype.has = listCacheHas;
  ListCache.prototype.set = listCacheSet;

  /**
   * Creates a map cache object to store key-value pairs.
   *
   * @private
   * @constructor
   * @param {Array} [entries] The key-value pairs to cache.
   */
  function MapCache(entries) {
    var index = -1,
      length = entries ? entries.length : 0;
    this.clear();
    while (++index < length) {
      var entry = entries[index];
      this.set(entry[0], entry[1]);
    }
  }

  /**
   * Removes all key-value entries from the map.
   *
   * @private
   * @name clear
   * @memberOf MapCache
   */
  function mapCacheClear() {
    this.__data__ = {
      'hash': new Hash(),
      'map': new (Map || ListCache)(),
      'string': new Hash()
    };
  }

  /**
   * Removes `key` and its value from the map.
   *
   * @private
   * @name delete
   * @memberOf MapCache
   * @param {string} key The key of the value to remove.
   * @returns {boolean} Returns `true` if the entry was removed, else `false`.
   */
  function mapCacheDelete(key) {
    return getMapData(this, key)['delete'](key);
  }

  /**
   * Gets the map value for `key`.
   *
   * @private
   * @name get
   * @memberOf MapCache
   * @param {string} key The key of the value to get.
   * @returns {*} Returns the entry value.
   */
  function mapCacheGet(key) {
    return getMapData(this, key).get(key);
  }

  /**
   * Checks if a map value for `key` exists.
   *
   * @private
   * @name has
   * @memberOf MapCache
   * @param {string} key The key of the entry to check.
   * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
   */
  function mapCacheHas(key) {
    return getMapData(this, key).has(key);
  }

  /**
   * Sets the map `key` to `value`.
   *
   * @private
   * @name set
   * @memberOf MapCache
   * @param {string} key The key of the value to set.
   * @param {*} value The value to set.
   * @returns {Object} Returns the map cache instance.
   */
  function mapCacheSet(key, value) {
    getMapData(this, key).set(key, value);
    return this;
  }

  // Add methods to `MapCache`.
  MapCache.prototype.clear = mapCacheClear;
  MapCache.prototype['delete'] = mapCacheDelete;
  MapCache.prototype.get = mapCacheGet;
  MapCache.prototype.has = mapCacheHas;
  MapCache.prototype.set = mapCacheSet;

  /**
   *
   * Creates an array cache object to store unique values.
   *
   * @private
   * @constructor
   * @param {Array} [values] The values to cache.
   */
  function SetCache(values) {
    var index = -1,
      length = values ? values.length : 0;
    this.__data__ = new MapCache();
    while (++index < length) {
      this.add(values[index]);
    }
  }

  /**
   * Adds `value` to the array cache.
   *
   * @private
   * @name add
   * @memberOf SetCache
   * @alias push
   * @param {*} value The value to cache.
   * @returns {Object} Returns the cache instance.
   */
  function setCacheAdd(value) {
    this.__data__.set(value, HASH_UNDEFINED);
    return this;
  }

  /**
   * Checks if `value` is in the array cache.
   *
   * @private
   * @name has
   * @memberOf SetCache
   * @param {*} value The value to search for.
   * @returns {number} Returns `true` if `value` is found, else `false`.
   */
  function setCacheHas(value) {
    return this.__data__.has(value);
  }

  // Add methods to `SetCache`.
  SetCache.prototype.add = SetCache.prototype.push = setCacheAdd;
  SetCache.prototype.has = setCacheHas;

  /**
   * Creates a stack cache object to store key-value pairs.
   *
   * @private
   * @constructor
   * @param {Array} [entries] The key-value pairs to cache.
   */
  function Stack(entries) {
    this.__data__ = new ListCache(entries);
  }

  /**
   * Removes all key-value entries from the stack.
   *
   * @private
   * @name clear
   * @memberOf Stack
   */
  function stackClear() {
    this.__data__ = new ListCache();
  }

  /**
   * Removes `key` and its value from the stack.
   *
   * @private
   * @name delete
   * @memberOf Stack
   * @param {string} key The key of the value to remove.
   * @returns {boolean} Returns `true` if the entry was removed, else `false`.
   */
  function stackDelete(key) {
    return this.__data__['delete'](key);
  }

  /**
   * Gets the stack value for `key`.
   *
   * @private
   * @name get
   * @memberOf Stack
   * @param {string} key The key of the value to get.
   * @returns {*} Returns the entry value.
   */
  function stackGet(key) {
    return this.__data__.get(key);
  }

  /**
   * Checks if a stack value for `key` exists.
   *
   * @private
   * @name has
   * @memberOf Stack
   * @param {string} key The key of the entry to check.
   * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
   */
  function stackHas(key) {
    return this.__data__.has(key);
  }

  /**
   * Sets the stack `key` to `value`.
   *
   * @private
   * @name set
   * @memberOf Stack
   * @param {string} key The key of the value to set.
   * @param {*} value The value to set.
   * @returns {Object} Returns the stack cache instance.
   */
  function stackSet(key, value) {
    var cache = this.__data__;
    if (cache instanceof ListCache) {
      var pairs = cache.__data__;
      if (!Map || pairs.length < LARGE_ARRAY_SIZE - 1) {
        pairs.push([key, value]);
        return this;
      }
      cache = this.__data__ = new MapCache(pairs);
    }
    cache.set(key, value);
    return this;
  }

  // Add methods to `Stack`.
  Stack.prototype.clear = stackClear;
  Stack.prototype['delete'] = stackDelete;
  Stack.prototype.get = stackGet;
  Stack.prototype.has = stackHas;
  Stack.prototype.set = stackSet;

  /**
   * Creates an array of the enumerable property names of the array-like `value`.
   *
   * @private
   * @param {*} value The value to query.
   * @param {boolean} inherited Specify returning inherited property names.
   * @returns {Array} Returns the array of property names.
   */
  function arrayLikeKeys(value, inherited) {
    // Safari 8.1 makes `arguments.callee` enumerable in strict mode.
    // Safari 9 makes `arguments.length` enumerable in strict mode.
    var result = isArray(value) || isArguments(value) ? baseTimes(value.length, String) : [];
    var length = result.length,
      skipIndexes = !!length;
    for (var key in value) {
      if ((inherited || hasOwnProperty.call(value, key)) && !(skipIndexes && (key == 'length' || isIndex(key, length)))) {
        result.push(key);
      }
    }
    return result;
  }

  /**
   * Gets the index at which the `key` is found in `array` of key-value pairs.
   *
   * @private
   * @param {Array} array The array to inspect.
   * @param {*} key The key to search for.
   * @returns {number} Returns the index of the matched value, else `-1`.
   */
  function assocIndexOf(array, key) {
    var length = array.length;
    while (length--) {
      if (eq(array[length][0], key)) {
        return length;
      }
    }
    return -1;
  }

  /**
   * Aggregates elements of `collection` on `accumulator` with keys transformed
   * by `iteratee` and values set by `setter`.
   *
   * @private
   * @param {Array|Object} collection The collection to iterate over.
   * @param {Function} setter The function to set `accumulator` values.
   * @param {Function} iteratee The iteratee to transform keys.
   * @param {Object} accumulator The initial aggregated object.
   * @returns {Function} Returns `accumulator`.
   */
  function baseAggregator(collection, setter, iteratee, accumulator) {
    baseEach(collection, function (value, key, collection) {
      setter(accumulator, value, iteratee(value), collection);
    });
    return accumulator;
  }

  /**
   * The base implementation of `_.forEach` without support for iteratee shorthands.
   *
   * @private
   * @param {Array|Object} collection The collection to iterate over.
   * @param {Function} iteratee The function invoked per iteration.
   * @returns {Array|Object} Returns `collection`.
   */
  var baseEach = createBaseEach(baseForOwn);

  /**
   * The base implementation of `baseForOwn` which iterates over `object`
   * properties returned by `keysFunc` and invokes `iteratee` for each property.
   * Iteratee functions may exit iteration early by explicitly returning `false`.
   *
   * @private
   * @param {Object} object The object to iterate over.
   * @param {Function} iteratee The function invoked per iteration.
   * @param {Function} keysFunc The function to get the keys of `object`.
   * @returns {Object} Returns `object`.
   */
  var baseFor = createBaseFor();

  /**
   * The base implementation of `_.forOwn` without support for iteratee shorthands.
   *
   * @private
   * @param {Object} object The object to iterate over.
   * @param {Function} iteratee The function invoked per iteration.
   * @returns {Object} Returns `object`.
   */
  function baseForOwn(object, iteratee) {
    return object && baseFor(object, iteratee, keys);
  }

  /**
   * The base implementation of `_.get` without support for default values.
   *
   * @private
   * @param {Object} object The object to query.
   * @param {Array|string} path The path of the property to get.
   * @returns {*} Returns the resolved value.
   */
  function baseGet(object, path) {
    path = isKey(path, object) ? [path] : castPath(path);
    var index = 0,
      length = path.length;
    while (object != null && index < length) {
      object = object[toKey(path[index++])];
    }
    return index && index == length ? object : undefined;
  }

  /**
   * The base implementation of `getTag`.
   *
   * @private
   * @param {*} value The value to query.
   * @returns {string} Returns the `toStringTag`.
   */
  function baseGetTag(value) {
    return objectToString.call(value);
  }

  /**
   * The base implementation of `_.hasIn` without support for deep paths.
   *
   * @private
   * @param {Object} [object] The object to query.
   * @param {Array|string} key The key to check.
   * @returns {boolean} Returns `true` if `key` exists, else `false`.
   */
  function baseHasIn(object, key) {
    return object != null && key in Object(object);
  }

  /**
   * The base implementation of `_.isEqual` which supports partial comparisons
   * and tracks traversed objects.
   *
   * @private
   * @param {*} value The value to compare.
   * @param {*} other The other value to compare.
   * @param {Function} [customizer] The function to customize comparisons.
   * @param {boolean} [bitmask] The bitmask of comparison flags.
   *  The bitmask may be composed of the following flags:
   *     1 - Unordered comparison
   *     2 - Partial comparison
   * @param {Object} [stack] Tracks traversed `value` and `other` objects.
   * @returns {boolean} Returns `true` if the values are equivalent, else `false`.
   */
  function baseIsEqual(value, other, customizer, bitmask, stack) {
    if (value === other) {
      return true;
    }
    if (value == null || other == null || !isObject(value) && !isObjectLike(other)) {
      return value !== value && other !== other;
    }
    return baseIsEqualDeep(value, other, baseIsEqual, customizer, bitmask, stack);
  }

  /**
   * A specialized version of `baseIsEqual` for arrays and objects which performs
   * deep comparisons and tracks traversed objects enabling objects with circular
   * references to be compared.
   *
   * @private
   * @param {Object} object The object to compare.
   * @param {Object} other The other object to compare.
   * @param {Function} equalFunc The function to determine equivalents of values.
   * @param {Function} [customizer] The function to customize comparisons.
   * @param {number} [bitmask] The bitmask of comparison flags. See `baseIsEqual`
   *  for more details.
   * @param {Object} [stack] Tracks traversed `object` and `other` objects.
   * @returns {boolean} Returns `true` if the objects are equivalent, else `false`.
   */
  function baseIsEqualDeep(object, other, equalFunc, customizer, bitmask, stack) {
    var objIsArr = isArray(object),
      othIsArr = isArray(other),
      objTag = arrayTag,
      othTag = arrayTag;
    if (!objIsArr) {
      objTag = getTag(object);
      objTag = objTag == argsTag ? objectTag : objTag;
    }
    if (!othIsArr) {
      othTag = getTag(other);
      othTag = othTag == argsTag ? objectTag : othTag;
    }
    var objIsObj = objTag == objectTag && !isHostObject(object),
      othIsObj = othTag == objectTag && !isHostObject(other),
      isSameTag = objTag == othTag;
    if (isSameTag && !objIsObj) {
      stack || (stack = new Stack());
      return objIsArr || isTypedArray(object) ? equalArrays(object, other, equalFunc, customizer, bitmask, stack) : equalByTag(object, other, objTag, equalFunc, customizer, bitmask, stack);
    }
    if (!(bitmask & PARTIAL_COMPARE_FLAG)) {
      var objIsWrapped = objIsObj && hasOwnProperty.call(object, '__wrapped__'),
        othIsWrapped = othIsObj && hasOwnProperty.call(other, '__wrapped__');
      if (objIsWrapped || othIsWrapped) {
        var objUnwrapped = objIsWrapped ? object.value() : object,
          othUnwrapped = othIsWrapped ? other.value() : other;
        stack || (stack = new Stack());
        return equalFunc(objUnwrapped, othUnwrapped, customizer, bitmask, stack);
      }
    }
    if (!isSameTag) {
      return false;
    }
    stack || (stack = new Stack());
    return equalObjects(object, other, equalFunc, customizer, bitmask, stack);
  }

  /**
   * The base implementation of `_.isMatch` without support for iteratee shorthands.
   *
   * @private
   * @param {Object} object The object to inspect.
   * @param {Object} source The object of property values to match.
   * @param {Array} matchData The property names, values, and compare flags to match.
   * @param {Function} [customizer] The function to customize comparisons.
   * @returns {boolean} Returns `true` if `object` is a match, else `false`.
   */
  function baseIsMatch(object, source, matchData, customizer) {
    var index = matchData.length,
      length = index,
      noCustomizer = !customizer;
    if (object == null) {
      return !length;
    }
    object = Object(object);
    while (index--) {
      var data = matchData[index];
      if (noCustomizer && data[2] ? data[1] !== object[data[0]] : !(data[0] in object)) {
        return false;
      }
    }
    while (++index < length) {
      data = matchData[index];
      var key = data[0],
        objValue = object[key],
        srcValue = data[1];
      if (noCustomizer && data[2]) {
        if (objValue === undefined && !(key in object)) {
          return false;
        }
      } else {
        var stack = new Stack();
        if (customizer) {
          var result = customizer(objValue, srcValue, key, object, source, stack);
        }
        if (!(result === undefined ? baseIsEqual(srcValue, objValue, customizer, UNORDERED_COMPARE_FLAG | PARTIAL_COMPARE_FLAG, stack) : result)) {
          return false;
        }
      }
    }
    return true;
  }

  /**
   * The base implementation of `_.isNative` without bad shim checks.
   *
   * @private
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is a native function,
   *  else `false`.
   */
  function baseIsNative(value) {
    if (!isObject(value) || isMasked(value)) {
      return false;
    }
    var pattern = isFunction(value) || isHostObject(value) ? reIsNative : reIsHostCtor;
    return pattern.test(toSource(value));
  }

  /**
   * The base implementation of `_.isTypedArray` without Node.js optimizations.
   *
   * @private
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is a typed array, else `false`.
   */
  function baseIsTypedArray(value) {
    return isObjectLike(value) && isLength(value.length) && !!typedArrayTags[objectToString.call(value)];
  }

  /**
   * The base implementation of `_.iteratee`.
   *
   * @private
   * @param {*} [value=_.identity] The value to convert to an iteratee.
   * @returns {Function} Returns the iteratee.
   */
  function baseIteratee(value) {
    // Don't store the `typeof` result in a variable to avoid a JIT bug in Safari 9.
    // See https://bugs.webkit.org/show_bug.cgi?id=156034 for more details.
    if (typeof value == 'function') {
      return value;
    }
    if (value == null) {
      return identity;
    }
    if (typeof value == 'object') {
      return isArray(value) ? baseMatchesProperty(value[0], value[1]) : baseMatches(value);
    }
    return property(value);
  }

  /**
   * The base implementation of `_.keys` which doesn't treat sparse arrays as dense.
   *
   * @private
   * @param {Object} object The object to query.
   * @returns {Array} Returns the array of property names.
   */
  function baseKeys(object) {
    if (!isPrototype(object)) {
      return nativeKeys(object);
    }
    var result = [];
    for (var key in Object(object)) {
      if (hasOwnProperty.call(object, key) && key != 'constructor') {
        result.push(key);
      }
    }
    return result;
  }

  /**
   * The base implementation of `_.matches` which doesn't clone `source`.
   *
   * @private
   * @param {Object} source The object of property values to match.
   * @returns {Function} Returns the new spec function.
   */
  function baseMatches(source) {
    var matchData = getMatchData(source);
    if (matchData.length == 1 && matchData[0][2]) {
      return matchesStrictComparable(matchData[0][0], matchData[0][1]);
    }
    return function (object) {
      return object === source || baseIsMatch(object, source, matchData);
    };
  }

  /**
   * The base implementation of `_.matchesProperty` which doesn't clone `srcValue`.
   *
   * @private
   * @param {string} path The path of the property to get.
   * @param {*} srcValue The value to match.
   * @returns {Function} Returns the new spec function.
   */
  function baseMatchesProperty(path, srcValue) {
    if (isKey(path) && isStrictComparable(srcValue)) {
      return matchesStrictComparable(toKey(path), srcValue);
    }
    return function (object) {
      var objValue = get(object, path);
      return objValue === undefined && objValue === srcValue ? hasIn(object, path) : baseIsEqual(srcValue, objValue, undefined, UNORDERED_COMPARE_FLAG | PARTIAL_COMPARE_FLAG);
    };
  }

  /**
   * A specialized version of `baseProperty` which supports deep paths.
   *
   * @private
   * @param {Array|string} path The path of the property to get.
   * @returns {Function} Returns the new accessor function.
   */
  function basePropertyDeep(path) {
    return function (object) {
      return baseGet(object, path);
    };
  }

  /**
   * The base implementation of `_.toString` which doesn't convert nullish
   * values to empty strings.
   *
   * @private
   * @param {*} value The value to process.
   * @returns {string} Returns the string.
   */
  function baseToString(value) {
    // Exit early for strings to avoid a performance hit in some environments.
    if (typeof value == 'string') {
      return value;
    }
    if (isSymbol(value)) {
      return symbolToString ? symbolToString.call(value) : '';
    }
    var result = value + '';
    return result == '0' && 1 / value == -INFINITY ? '-0' : result;
  }

  /**
   * Casts `value` to a path array if it's not one.
   *
   * @private
   * @param {*} value The value to inspect.
   * @returns {Array} Returns the cast property path array.
   */
  function castPath(value) {
    return isArray(value) ? value : stringToPath(value);
  }

  /**
   * Creates a function like `_.groupBy`.
   *
   * @private
   * @param {Function} setter The function to set accumulator values.
   * @param {Function} [initializer] The accumulator object initializer.
   * @returns {Function} Returns the new aggregator function.
   */
  function createAggregator(setter, initializer) {
    return function (collection, iteratee) {
      var func = isArray(collection) ? arrayAggregator : baseAggregator,
        accumulator = initializer ? initializer() : {};
      return func(collection, setter, baseIteratee(iteratee), accumulator);
    };
  }

  /**
   * Creates a `baseEach` or `baseEachRight` function.
   *
   * @private
   * @param {Function} eachFunc The function to iterate over a collection.
   * @param {boolean} [fromRight] Specify iterating from right to left.
   * @returns {Function} Returns the new base function.
   */
  function createBaseEach(eachFunc, fromRight) {
    return function (collection, iteratee) {
      if (collection == null) {
        return collection;
      }
      if (!isArrayLike(collection)) {
        return eachFunc(collection, iteratee);
      }
      var length = collection.length,
        index = fromRight ? length : -1,
        iterable = Object(collection);
      while (fromRight ? index-- : ++index < length) {
        if (iteratee(iterable[index], index, iterable) === false) {
          break;
        }
      }
      return collection;
    };
  }

  /**
   * Creates a base function for methods like `_.forIn` and `_.forOwn`.
   *
   * @private
   * @param {boolean} [fromRight] Specify iterating from right to left.
   * @returns {Function} Returns the new base function.
   */
  function createBaseFor(fromRight) {
    return function (object, iteratee, keysFunc) {
      var index = -1,
        iterable = Object(object),
        props = keysFunc(object),
        length = props.length;
      while (length--) {
        var key = props[fromRight ? length : ++index];
        if (iteratee(iterable[key], key, iterable) === false) {
          break;
        }
      }
      return object;
    };
  }

  /**
   * A specialized version of `baseIsEqualDeep` for arrays with support for
   * partial deep comparisons.
   *
   * @private
   * @param {Array} array The array to compare.
   * @param {Array} other The other array to compare.
   * @param {Function} equalFunc The function to determine equivalents of values.
   * @param {Function} customizer The function to customize comparisons.
   * @param {number} bitmask The bitmask of comparison flags. See `baseIsEqual`
   *  for more details.
   * @param {Object} stack Tracks traversed `array` and `other` objects.
   * @returns {boolean} Returns `true` if the arrays are equivalent, else `false`.
   */
  function equalArrays(array, other, equalFunc, customizer, bitmask, stack) {
    var isPartial = bitmask & PARTIAL_COMPARE_FLAG,
      arrLength = array.length,
      othLength = other.length;
    if (arrLength != othLength && !(isPartial && othLength > arrLength)) {
      return false;
    }
    // Assume cyclic values are equal.
    var stacked = stack.get(array);
    if (stacked && stack.get(other)) {
      return stacked == other;
    }
    var index = -1,
      result = true,
      seen = bitmask & UNORDERED_COMPARE_FLAG ? new SetCache() : undefined;
    stack.set(array, other);
    stack.set(other, array);

    // Ignore non-index properties.
    while (++index < arrLength) {
      var arrValue = array[index],
        othValue = other[index];
      if (customizer) {
        var compared = isPartial ? customizer(othValue, arrValue, index, other, array, stack) : customizer(arrValue, othValue, index, array, other, stack);
      }
      if (compared !== undefined) {
        if (compared) {
          continue;
        }
        result = false;
        break;
      }
      // Recursively compare arrays (susceptible to call stack limits).
      if (seen) {
        if (!arraySome(other, function (othValue, othIndex) {
          if (!seen.has(othIndex) && (arrValue === othValue || equalFunc(arrValue, othValue, customizer, bitmask, stack))) {
            return seen.add(othIndex);
          }
        })) {
          result = false;
          break;
        }
      } else if (!(arrValue === othValue || equalFunc(arrValue, othValue, customizer, bitmask, stack))) {
        result = false;
        break;
      }
    }
    stack['delete'](array);
    stack['delete'](other);
    return result;
  }

  /**
   * A specialized version of `baseIsEqualDeep` for comparing objects of
   * the same `toStringTag`.
   *
   * **Note:** This function only supports comparing values with tags of
   * `Boolean`, `Date`, `Error`, `Number`, `RegExp`, or `String`.
   *
   * @private
   * @param {Object} object The object to compare.
   * @param {Object} other The other object to compare.
   * @param {string} tag The `toStringTag` of the objects to compare.
   * @param {Function} equalFunc The function to determine equivalents of values.
   * @param {Function} customizer The function to customize comparisons.
   * @param {number} bitmask The bitmask of comparison flags. See `baseIsEqual`
   *  for more details.
   * @param {Object} stack Tracks traversed `object` and `other` objects.
   * @returns {boolean} Returns `true` if the objects are equivalent, else `false`.
   */
  function equalByTag(object, other, tag, equalFunc, customizer, bitmask, stack) {
    switch (tag) {
      case dataViewTag:
        if (object.byteLength != other.byteLength || object.byteOffset != other.byteOffset) {
          return false;
        }
        object = object.buffer;
        other = other.buffer;
      case arrayBufferTag:
        if (object.byteLength != other.byteLength || !equalFunc(new Uint8Array(object), new Uint8Array(other))) {
          return false;
        }
        return true;
      case boolTag:
      case dateTag:
      case numberTag:
        // Coerce booleans to `1` or `0` and dates to milliseconds.
        // Invalid dates are coerced to `NaN`.
        return eq(+object, +other);
      case errorTag:
        return object.name == other.name && object.message == other.message;
      case regexpTag:
      case stringTag:
        // Coerce regexes to strings and treat strings, primitives and objects,
        // as equal. See http://www.ecma-international.org/ecma-262/7.0/#sec-regexp.prototype.tostring
        // for more details.
        return object == other + '';
      case mapTag:
        var convert = mapToArray;
      case setTag:
        var isPartial = bitmask & PARTIAL_COMPARE_FLAG;
        convert || (convert = setToArray);
        if (object.size != other.size && !isPartial) {
          return false;
        }
        // Assume cyclic values are equal.
        var stacked = stack.get(object);
        if (stacked) {
          return stacked == other;
        }
        bitmask |= UNORDERED_COMPARE_FLAG;

        // Recursively compare objects (susceptible to call stack limits).
        stack.set(object, other);
        var result = equalArrays(convert(object), convert(other), equalFunc, customizer, bitmask, stack);
        stack['delete'](object);
        return result;
      case symbolTag:
        if (symbolValueOf) {
          return symbolValueOf.call(object) == symbolValueOf.call(other);
        }
    }
    return false;
  }

  /**
   * A specialized version of `baseIsEqualDeep` for objects with support for
   * partial deep comparisons.
   *
   * @private
   * @param {Object} object The object to compare.
   * @param {Object} other The other object to compare.
   * @param {Function} equalFunc The function to determine equivalents of values.
   * @param {Function} customizer The function to customize comparisons.
   * @param {number} bitmask The bitmask of comparison flags. See `baseIsEqual`
   *  for more details.
   * @param {Object} stack Tracks traversed `object` and `other` objects.
   * @returns {boolean} Returns `true` if the objects are equivalent, else `false`.
   */
  function equalObjects(object, other, equalFunc, customizer, bitmask, stack) {
    var isPartial = bitmask & PARTIAL_COMPARE_FLAG,
      objProps = keys(object),
      objLength = objProps.length,
      othProps = keys(other),
      othLength = othProps.length;
    if (objLength != othLength && !isPartial) {
      return false;
    }
    var index = objLength;
    while (index--) {
      var key = objProps[index];
      if (!(isPartial ? key in other : hasOwnProperty.call(other, key))) {
        return false;
      }
    }
    // Assume cyclic values are equal.
    var stacked = stack.get(object);
    if (stacked && stack.get(other)) {
      return stacked == other;
    }
    var result = true;
    stack.set(object, other);
    stack.set(other, object);
    var skipCtor = isPartial;
    while (++index < objLength) {
      key = objProps[index];
      var objValue = object[key],
        othValue = other[key];
      if (customizer) {
        var compared = isPartial ? customizer(othValue, objValue, key, other, object, stack) : customizer(objValue, othValue, key, object, other, stack);
      }
      // Recursively compare objects (susceptible to call stack limits).
      if (!(compared === undefined ? objValue === othValue || equalFunc(objValue, othValue, customizer, bitmask, stack) : compared)) {
        result = false;
        break;
      }
      skipCtor || (skipCtor = key == 'constructor');
    }
    if (result && !skipCtor) {
      var objCtor = object.constructor,
        othCtor = other.constructor;

      // Non `Object` object instances with different constructors are not equal.
      if (objCtor != othCtor && 'constructor' in object && 'constructor' in other && !(typeof objCtor == 'function' && objCtor instanceof objCtor && typeof othCtor == 'function' && othCtor instanceof othCtor)) {
        result = false;
      }
    }
    stack['delete'](object);
    stack['delete'](other);
    return result;
  }

  /**
   * Gets the data for `map`.
   *
   * @private
   * @param {Object} map The map to query.
   * @param {string} key The reference key.
   * @returns {*} Returns the map data.
   */
  function getMapData(map, key) {
    var data = map.__data__;
    return isKeyable(key) ? data[typeof key == 'string' ? 'string' : 'hash'] : data.map;
  }

  /**
   * Gets the property names, values, and compare flags of `object`.
   *
   * @private
   * @param {Object} object The object to query.
   * @returns {Array} Returns the match data of `object`.
   */
  function getMatchData(object) {
    var result = keys(object),
      length = result.length;
    while (length--) {
      var key = result[length],
        value = object[key];
      result[length] = [key, value, isStrictComparable(value)];
    }
    return result;
  }

  /**
   * Gets the native function at `key` of `object`.
   *
   * @private
   * @param {Object} object The object to query.
   * @param {string} key The key of the method to get.
   * @returns {*} Returns the function if it's native, else `undefined`.
   */
  function getNative(object, key) {
    var value = getValue(object, key);
    return baseIsNative(value) ? value : undefined;
  }

  /**
   * Gets the `toStringTag` of `value`.
   *
   * @private
   * @param {*} value The value to query.
   * @returns {string} Returns the `toStringTag`.
   */
  var getTag = baseGetTag;

  // Fallback for data views, maps, sets, and weak maps in IE 11,
  // for data views in Edge < 14, and promises in Node.js.
  if (DataView && getTag(new DataView(new ArrayBuffer(1))) != dataViewTag || Map && getTag(new Map()) != mapTag || Promise && getTag(Promise.resolve()) != promiseTag || Set && getTag(new Set()) != setTag || WeakMap && getTag(new WeakMap()) != weakMapTag) {
    getTag = function (value) {
      var result = objectToString.call(value),
        Ctor = result == objectTag ? value.constructor : undefined,
        ctorString = Ctor ? toSource(Ctor) : undefined;
      if (ctorString) {
        switch (ctorString) {
          case dataViewCtorString:
            return dataViewTag;
          case mapCtorString:
            return mapTag;
          case promiseCtorString:
            return promiseTag;
          case setCtorString:
            return setTag;
          case weakMapCtorString:
            return weakMapTag;
        }
      }
      return result;
    };
  }

  /**
   * Checks if `path` exists on `object`.
   *
   * @private
   * @param {Object} object The object to query.
   * @param {Array|string} path The path to check.
   * @param {Function} hasFunc The function to check properties.
   * @returns {boolean} Returns `true` if `path` exists, else `false`.
   */
  function hasPath(object, path, hasFunc) {
    path = isKey(path, object) ? [path] : castPath(path);
    var result,
      index = -1,
      length = path.length;
    while (++index < length) {
      var key = toKey(path[index]);
      if (!(result = object != null && hasFunc(object, key))) {
        break;
      }
      object = object[key];
    }
    if (result) {
      return result;
    }
    var length = object ? object.length : 0;
    return !!length && isLength(length) && isIndex(key, length) && (isArray(object) || isArguments(object));
  }

  /**
   * Checks if `value` is a valid array-like index.
   *
   * @private
   * @param {*} value The value to check.
   * @param {number} [length=MAX_SAFE_INTEGER] The upper bounds of a valid index.
   * @returns {boolean} Returns `true` if `value` is a valid index, else `false`.
   */
  function isIndex(value, length) {
    length = length == null ? MAX_SAFE_INTEGER : length;
    return !!length && (typeof value == 'number' || reIsUint.test(value)) && value > -1 && value % 1 == 0 && value < length;
  }

  /**
   * Checks if `value` is a property name and not a property path.
   *
   * @private
   * @param {*} value The value to check.
   * @param {Object} [object] The object to query keys on.
   * @returns {boolean} Returns `true` if `value` is a property name, else `false`.
   */
  function isKey(value, object) {
    if (isArray(value)) {
      return false;
    }
    var type = typeof value;
    if (type == 'number' || type == 'symbol' || type == 'boolean' || value == null || isSymbol(value)) {
      return true;
    }
    return reIsPlainProp.test(value) || !reIsDeepProp.test(value) || object != null && value in Object(object);
  }

  /**
   * Checks if `value` is suitable for use as unique object key.
   *
   * @private
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is suitable, else `false`.
   */
  function isKeyable(value) {
    var type = typeof value;
    return type == 'string' || type == 'number' || type == 'symbol' || type == 'boolean' ? value !== '__proto__' : value === null;
  }

  /**
   * Checks if `func` has its source masked.
   *
   * @private
   * @param {Function} func The function to check.
   * @returns {boolean} Returns `true` if `func` is masked, else `false`.
   */
  function isMasked(func) {
    return !!maskSrcKey && maskSrcKey in func;
  }

  /**
   * Checks if `value` is likely a prototype object.
   *
   * @private
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is a prototype, else `false`.
   */
  function isPrototype(value) {
    var Ctor = value && value.constructor,
      proto = typeof Ctor == 'function' && Ctor.prototype || objectProto;
    return value === proto;
  }

  /**
   * Checks if `value` is suitable for strict equality comparisons, i.e. `===`.
   *
   * @private
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` if suitable for strict
   *  equality comparisons, else `false`.
   */
  function isStrictComparable(value) {
    return value === value && !isObject(value);
  }

  /**
   * A specialized version of `matchesProperty` for source values suitable
   * for strict equality comparisons, i.e. `===`.
   *
   * @private
   * @param {string} key The key of the property to get.
   * @param {*} srcValue The value to match.
   * @returns {Function} Returns the new spec function.
   */
  function matchesStrictComparable(key, srcValue) {
    return function (object) {
      if (object == null) {
        return false;
      }
      return object[key] === srcValue && (srcValue !== undefined || key in Object(object));
    };
  }

  /**
   * Converts `string` to a property path array.
   *
   * @private
   * @param {string} string The string to convert.
   * @returns {Array} Returns the property path array.
   */
  var stringToPath = memoize(function (string) {
    string = toString(string);
    var result = [];
    if (reLeadingDot.test(string)) {
      result.push('');
    }
    string.replace(rePropName, function (match, number, quote, string) {
      result.push(quote ? string.replace(reEscapeChar, '$1') : number || match);
    });
    return result;
  });

  /**
   * Converts `value` to a string key if it's not a string or symbol.
   *
   * @private
   * @param {*} value The value to inspect.
   * @returns {string|symbol} Returns the key.
   */
  function toKey(value) {
    if (typeof value == 'string' || isSymbol(value)) {
      return value;
    }
    var result = value + '';
    return result == '0' && 1 / value == -INFINITY ? '-0' : result;
  }

  /**
   * Converts `func` to its source code.
   *
   * @private
   * @param {Function} func The function to process.
   * @returns {string} Returns the source code.
   */
  function toSource(func) {
    if (func != null) {
      try {
        return funcToString.call(func);
      } catch (e) {}
      try {
        return func + '';
      } catch (e) {}
    }
    return '';
  }

  /**
   * Creates an object composed of keys generated from the results of running
   * each element of `collection` thru `iteratee`. The order of grouped values
   * is determined by the order they occur in `collection`. The corresponding
   * value of each key is an array of elements responsible for generating the
   * key. The iteratee is invoked with one argument: (value).
   *
   * @static
   * @memberOf _
   * @since 0.1.0
   * @category Collection
   * @param {Array|Object} collection The collection to iterate over.
   * @param {Function} [iteratee=_.identity]
   *  The iteratee to transform keys.
   * @returns {Object} Returns the composed aggregate object.
   * @example
   *
   * _.groupBy([6.1, 4.2, 6.3], Math.floor);
   * // => { '4': [4.2], '6': [6.1, 6.3] }
   *
   * // The `_.property` iteratee shorthand.
   * _.groupBy(['one', 'two', 'three'], 'length');
   * // => { '3': ['one', 'two'], '5': ['three'] }
   */
  var groupBy = createAggregator(function (result, value, key) {
    if (hasOwnProperty.call(result, key)) {
      result[key].push(value);
    } else {
      result[key] = [value];
    }
  });

  /**
   * Creates a function that memoizes the result of `func`. If `resolver` is
   * provided, it determines the cache key for storing the result based on the
   * arguments provided to the memoized function. By default, the first argument
   * provided to the memoized function is used as the map cache key. The `func`
   * is invoked with the `this` binding of the memoized function.
   *
   * **Note:** The cache is exposed as the `cache` property on the memoized
   * function. Its creation may be customized by replacing the `_.memoize.Cache`
   * constructor with one whose instances implement the
   * [`Map`](http://ecma-international.org/ecma-262/7.0/#sec-properties-of-the-map-prototype-object)
   * method interface of `delete`, `get`, `has`, and `set`.
   *
   * @static
   * @memberOf _
   * @since 0.1.0
   * @category Function
   * @param {Function} func The function to have its output memoized.
   * @param {Function} [resolver] The function to resolve the cache key.
   * @returns {Function} Returns the new memoized function.
   * @example
   *
   * var object = { 'a': 1, 'b': 2 };
   * var other = { 'c': 3, 'd': 4 };
   *
   * var values = _.memoize(_.values);
   * values(object);
   * // => [1, 2]
   *
   * values(other);
   * // => [3, 4]
   *
   * object.a = 2;
   * values(object);
   * // => [1, 2]
   *
   * // Modify the result cache.
   * values.cache.set(object, ['a', 'b']);
   * values(object);
   * // => ['a', 'b']
   *
   * // Replace `_.memoize.Cache`.
   * _.memoize.Cache = WeakMap;
   */
  function memoize(func, resolver) {
    if (typeof func != 'function' || resolver && typeof resolver != 'function') {
      throw new TypeError(FUNC_ERROR_TEXT);
    }
    var memoized = function () {
      var args = arguments,
        key = resolver ? resolver.apply(this, args) : args[0],
        cache = memoized.cache;
      if (cache.has(key)) {
        return cache.get(key);
      }
      var result = func.apply(this, args);
      memoized.cache = cache.set(key, result);
      return result;
    };
    memoized.cache = new (memoize.Cache || MapCache)();
    return memoized;
  }

  // Assign cache to `_.memoize`.
  memoize.Cache = MapCache;

  /**
   * Performs a
   * [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
   * comparison between two values to determine if they are equivalent.
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category Lang
   * @param {*} value The value to compare.
   * @param {*} other The other value to compare.
   * @returns {boolean} Returns `true` if the values are equivalent, else `false`.
   * @example
   *
   * var object = { 'a': 1 };
   * var other = { 'a': 1 };
   *
   * _.eq(object, object);
   * // => true
   *
   * _.eq(object, other);
   * // => false
   *
   * _.eq('a', 'a');
   * // => true
   *
   * _.eq('a', Object('a'));
   * // => false
   *
   * _.eq(NaN, NaN);
   * // => true
   */
  function eq(value, other) {
    return value === other || value !== value && other !== other;
  }

  /**
   * Checks if `value` is likely an `arguments` object.
   *
   * @static
   * @memberOf _
   * @since 0.1.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is an `arguments` object,
   *  else `false`.
   * @example
   *
   * _.isArguments(function() { return arguments; }());
   * // => true
   *
   * _.isArguments([1, 2, 3]);
   * // => false
   */
  function isArguments(value) {
    // Safari 8.1 makes `arguments.callee` enumerable in strict mode.
    return isArrayLikeObject(value) && hasOwnProperty.call(value, 'callee') && (!propertyIsEnumerable.call(value, 'callee') || objectToString.call(value) == argsTag);
  }

  /**
   * Checks if `value` is classified as an `Array` object.
   *
   * @static
   * @memberOf _
   * @since 0.1.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is an array, else `false`.
   * @example
   *
   * _.isArray([1, 2, 3]);
   * // => true
   *
   * _.isArray(document.body.children);
   * // => false
   *
   * _.isArray('abc');
   * // => false
   *
   * _.isArray(_.noop);
   * // => false
   */
  var isArray = Array.isArray;

  /**
   * Checks if `value` is array-like. A value is considered array-like if it's
   * not a function and has a `value.length` that's an integer greater than or
   * equal to `0` and less than or equal to `Number.MAX_SAFE_INTEGER`.
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is array-like, else `false`.
   * @example
   *
   * _.isArrayLike([1, 2, 3]);
   * // => true
   *
   * _.isArrayLike(document.body.children);
   * // => true
   *
   * _.isArrayLike('abc');
   * // => true
   *
   * _.isArrayLike(_.noop);
   * // => false
   */
  function isArrayLike(value) {
    return value != null && isLength(value.length) && !isFunction(value);
  }

  /**
   * This method is like `_.isArrayLike` except that it also checks if `value`
   * is an object.
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is an array-like object,
   *  else `false`.
   * @example
   *
   * _.isArrayLikeObject([1, 2, 3]);
   * // => true
   *
   * _.isArrayLikeObject(document.body.children);
   * // => true
   *
   * _.isArrayLikeObject('abc');
   * // => false
   *
   * _.isArrayLikeObject(_.noop);
   * // => false
   */
  function isArrayLikeObject(value) {
    return isObjectLike(value) && isArrayLike(value);
  }

  /**
   * Checks if `value` is classified as a `Function` object.
   *
   * @static
   * @memberOf _
   * @since 0.1.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is a function, else `false`.
   * @example
   *
   * _.isFunction(_);
   * // => true
   *
   * _.isFunction(/abc/);
   * // => false
   */
  function isFunction(value) {
    // The use of `Object#toString` avoids issues with the `typeof` operator
    // in Safari 8-9 which returns 'object' for typed array and other constructors.
    var tag = isObject(value) ? objectToString.call(value) : '';
    return tag == funcTag || tag == genTag;
  }

  /**
   * Checks if `value` is a valid array-like length.
   *
   * **Note:** This method is loosely based on
   * [`ToLength`](http://ecma-international.org/ecma-262/7.0/#sec-tolength).
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is a valid length, else `false`.
   * @example
   *
   * _.isLength(3);
   * // => true
   *
   * _.isLength(Number.MIN_VALUE);
   * // => false
   *
   * _.isLength(Infinity);
   * // => false
   *
   * _.isLength('3');
   * // => false
   */
  function isLength(value) {
    return typeof value == 'number' && value > -1 && value % 1 == 0 && value <= MAX_SAFE_INTEGER;
  }

  /**
   * Checks if `value` is the
   * [language type](http://www.ecma-international.org/ecma-262/7.0/#sec-ecmascript-language-types)
   * of `Object`. (e.g. arrays, functions, objects, regexes, `new Number(0)`, and `new String('')`)
   *
   * @static
   * @memberOf _
   * @since 0.1.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is an object, else `false`.
   * @example
   *
   * _.isObject({});
   * // => true
   *
   * _.isObject([1, 2, 3]);
   * // => true
   *
   * _.isObject(_.noop);
   * // => true
   *
   * _.isObject(null);
   * // => false
   */
  function isObject(value) {
    var type = typeof value;
    return !!value && (type == 'object' || type == 'function');
  }

  /**
   * Checks if `value` is object-like. A value is object-like if it's not `null`
   * and has a `typeof` result of "object".
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is object-like, else `false`.
   * @example
   *
   * _.isObjectLike({});
   * // => true
   *
   * _.isObjectLike([1, 2, 3]);
   * // => true
   *
   * _.isObjectLike(_.noop);
   * // => false
   *
   * _.isObjectLike(null);
   * // => false
   */
  function isObjectLike(value) {
    return !!value && typeof value == 'object';
  }

  /**
   * Checks if `value` is classified as a `Symbol` primitive or object.
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is a symbol, else `false`.
   * @example
   *
   * _.isSymbol(Symbol.iterator);
   * // => true
   *
   * _.isSymbol('abc');
   * // => false
   */
  function isSymbol(value) {
    return typeof value == 'symbol' || isObjectLike(value) && objectToString.call(value) == symbolTag;
  }

  /**
   * Checks if `value` is classified as a typed array.
   *
   * @static
   * @memberOf _
   * @since 3.0.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is a typed array, else `false`.
   * @example
   *
   * _.isTypedArray(new Uint8Array);
   * // => true
   *
   * _.isTypedArray([]);
   * // => false
   */
  var isTypedArray = nodeIsTypedArray ? baseUnary(nodeIsTypedArray) : baseIsTypedArray;

  /**
   * Converts `value` to a string. An empty string is returned for `null`
   * and `undefined` values. The sign of `-0` is preserved.
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category Lang
   * @param {*} value The value to process.
   * @returns {string} Returns the string.
   * @example
   *
   * _.toString(null);
   * // => ''
   *
   * _.toString(-0);
   * // => '-0'
   *
   * _.toString([1, 2, 3]);
   * // => '1,2,3'
   */
  function toString(value) {
    return value == null ? '' : baseToString(value);
  }

  /**
   * Gets the value at `path` of `object`. If the resolved value is
   * `undefined`, the `defaultValue` is returned in its place.
   *
   * @static
   * @memberOf _
   * @since 3.7.0
   * @category Object
   * @param {Object} object The object to query.
   * @param {Array|string} path The path of the property to get.
   * @param {*} [defaultValue] The value returned for `undefined` resolved values.
   * @returns {*} Returns the resolved value.
   * @example
   *
   * var object = { 'a': [{ 'b': { 'c': 3 } }] };
   *
   * _.get(object, 'a[0].b.c');
   * // => 3
   *
   * _.get(object, ['a', '0', 'b', 'c']);
   * // => 3
   *
   * _.get(object, 'a.b.c', 'default');
   * // => 'default'
   */
  function get(object, path, defaultValue) {
    var result = object == null ? undefined : baseGet(object, path);
    return result === undefined ? defaultValue : result;
  }

  /**
   * Checks if `path` is a direct or inherited property of `object`.
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category Object
   * @param {Object} object The object to query.
   * @param {Array|string} path The path to check.
   * @returns {boolean} Returns `true` if `path` exists, else `false`.
   * @example
   *
   * var object = _.create({ 'a': _.create({ 'b': 2 }) });
   *
   * _.hasIn(object, 'a');
   * // => true
   *
   * _.hasIn(object, 'a.b');
   * // => true
   *
   * _.hasIn(object, ['a', 'b']);
   * // => true
   *
   * _.hasIn(object, 'b');
   * // => false
   */
  function hasIn(object, path) {
    return object != null && hasPath(object, path, baseHasIn);
  }

  /**
   * Creates an array of the own enumerable property names of `object`.
   *
   * **Note:** Non-object values are coerced to objects. See the
   * [ES spec](http://ecma-international.org/ecma-262/7.0/#sec-object.keys)
   * for more details.
   *
   * @static
   * @since 0.1.0
   * @memberOf _
   * @category Object
   * @param {Object} object The object to query.
   * @returns {Array} Returns the array of property names.
   * @example
   *
   * function Foo() {
   *   this.a = 1;
   *   this.b = 2;
   * }
   *
   * Foo.prototype.c = 3;
   *
   * _.keys(new Foo);
   * // => ['a', 'b'] (iteration order is not guaranteed)
   *
   * _.keys('hi');
   * // => ['0', '1']
   */
  function keys(object) {
    return isArrayLike(object) ? arrayLikeKeys(object) : baseKeys(object);
  }

  /**
   * This method returns the first argument it receives.
   *
   * @static
   * @since 0.1.0
   * @memberOf _
   * @category Util
   * @param {*} value Any value.
   * @returns {*} Returns `value`.
   * @example
   *
   * var object = { 'a': 1 };
   *
   * console.log(_.identity(object) === object);
   * // => true
   */
  function identity(value) {
    return value;
  }

  /**
   * Creates a function that returns the value at `path` of a given object.
   *
   * @static
   * @memberOf _
   * @since 2.4.0
   * @category Util
   * @param {Array|string} path The path of the property to get.
   * @returns {Function} Returns the new accessor function.
   * @example
   *
   * var objects = [
   *   { 'a': { 'b': 2 } },
   *   { 'a': { 'b': 1 } }
   * ];
   *
   * _.map(objects, _.property('a.b'));
   * // => [2, 1]
   *
   * _.map(_.sortBy(objects, _.property(['a', 'b'])), 'a.b');
   * // => [1, 2]
   */
  function property(path) {
    return isKey(path) ? baseProperty(toKey(path)) : basePropertyDeep(path);
  }
  module.exports = groupBy;
})(lodash_groupby, lodash_groupby.exports);
var groupBy = lodash_groupby.exports;

/**
 * lodash (Custom Build) <https://lodash.com/>
 * Build: `lodash modularize exports="npm" -o ./`
 * Copyright jQuery Foundation and other contributors <https://jquery.org/>
 * Released under MIT license <https://lodash.com/license>
 * Based on Underscore.js 1.8.3 <http://underscorejs.org/LICENSE>
 * Copyright Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
 */

/** Used as references for various `Number` constants. */
var MAX_SAFE_INTEGER = 9007199254740991;

/** `Object#toString` result references. */
var argsTag = '[object Arguments]',
  funcTag = '[object Function]',
  genTag = '[object GeneratorFunction]',
  mapTag = '[object Map]',
  objectTag = '[object Object]',
  promiseTag = '[object Promise]',
  setTag = '[object Set]',
  weakMapTag = '[object WeakMap]';
var dataViewTag = '[object DataView]';

/**
 * Used to match `RegExp`
 * [syntax characters](http://ecma-international.org/ecma-262/7.0/#sec-patterns).
 */
var reRegExpChar = /[\\^$.*+?()[\]{}|]/g;

/** Used to detect host constructors (Safari). */
var reIsHostCtor = /^\[object .+?Constructor\]$/;

/** Used to detect unsigned integer values. */
var reIsUint = /^(?:0|[1-9]\d*)$/;

/** Detect free variable `global` from Node.js. */
var freeGlobal$1 = typeof commonjsGlobal == 'object' && commonjsGlobal && commonjsGlobal.Object === Object && commonjsGlobal;

/** Detect free variable `self`. */
var freeSelf$1 = typeof self == 'object' && self && self.Object === Object && self;

/** Used as a reference to the global object. */
var root$1 = freeGlobal$1 || freeSelf$1 || Function('return this')();

/**
 * A specialized version of `_.map` for arrays without support for iteratee
 * shorthands.
 *
 * @private
 * @param {Array} [array] The array to iterate over.
 * @param {Function} iteratee The function invoked per iteration.
 * @returns {Array} Returns the new mapped array.
 */
function arrayMap(array, iteratee) {
  var index = -1,
    length = array ? array.length : 0,
    result = Array(length);
  while (++index < length) {
    result[index] = iteratee(array[index], index, array);
  }
  return result;
}

/**
 * The base implementation of `_.times` without support for iteratee shorthands
 * or max array length checks.
 *
 * @private
 * @param {number} n The number of times to invoke `iteratee`.
 * @param {Function} iteratee The function invoked per iteration.
 * @returns {Array} Returns the array of results.
 */
function baseTimes(n, iteratee) {
  var index = -1,
    result = Array(n);
  while (++index < n) {
    result[index] = iteratee(index);
  }
  return result;
}

/**
 * The base implementation of `_.toPairs` and `_.toPairsIn` which creates an array
 * of key-value pairs for `object` corresponding to the property names of `props`.
 *
 * @private
 * @param {Object} object The object to query.
 * @param {Array} props The property names to get values for.
 * @returns {Object} Returns the key-value pairs.
 */
function baseToPairs(object, props) {
  return arrayMap(props, function (key) {
    return [key, object[key]];
  });
}

/**
 * Gets the value at `key` of `object`.
 *
 * @private
 * @param {Object} [object] The object to query.
 * @param {string} key The key of the property to get.
 * @returns {*} Returns the property value.
 */
function getValue(object, key) {
  return object == null ? undefined : object[key];
}

/**
 * Checks if `value` is a host object in IE < 9.
 *
 * @private
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is a host object, else `false`.
 */
function isHostObject(value) {
  // Many host objects are `Object` objects that can coerce to strings
  // despite having improperly defined `toString` methods.
  var result = false;
  if (value != null && typeof value.toString != 'function') {
    try {
      result = !!(value + '');
    } catch (e) {}
  }
  return result;
}

/**
 * Converts `map` to its key-value pairs.
 *
 * @private
 * @param {Object} map The map to convert.
 * @returns {Array} Returns the key-value pairs.
 */
function mapToArray(map) {
  var index = -1,
    result = Array(map.size);
  map.forEach(function (value, key) {
    result[++index] = [key, value];
  });
  return result;
}

/**
 * Creates a unary function that invokes `func` with its argument transformed.
 *
 * @private
 * @param {Function} func The function to wrap.
 * @param {Function} transform The argument transform.
 * @returns {Function} Returns the new function.
 */
function overArg(func, transform) {
  return function (arg) {
    return func(transform(arg));
  };
}

/**
 * Converts `set` to its value-value pairs.
 *
 * @private
 * @param {Object} set The set to convert.
 * @returns {Array} Returns the value-value pairs.
 */
function setToPairs(set) {
  var index = -1,
    result = Array(set.size);
  set.forEach(function (value) {
    result[++index] = [value, value];
  });
  return result;
}

/** Used for built-in method references. */
var funcProto = Function.prototype,
  objectProto$1 = Object.prototype;

/** Used to detect overreaching core-js shims. */
var coreJsData = root$1['__core-js_shared__'];

/** Used to detect methods masquerading as native. */
var maskSrcKey = function () {
  var uid = /[^.]+$/.exec(coreJsData && coreJsData.keys && coreJsData.keys.IE_PROTO || '');
  return uid ? 'Symbol(src)_1.' + uid : '';
}();

/** Used to resolve the decompiled source of functions. */
var funcToString = funcProto.toString;

/** Used to check objects for own properties. */
var hasOwnProperty = objectProto$1.hasOwnProperty;

/**
 * Used to resolve the
 * [`toStringTag`](http://ecma-international.org/ecma-262/7.0/#sec-object.prototype.tostring)
 * of values.
 */
var objectToString$1 = objectProto$1.toString;

/** Used to detect if a method is native. */
var reIsNative = RegExp('^' + funcToString.call(hasOwnProperty).replace(reRegExpChar, '\\$&').replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g, '$1.*?') + '$');

/** Built-in value references. */
var propertyIsEnumerable = objectProto$1.propertyIsEnumerable;

/* Built-in method references for those with the same name as other `lodash` methods. */
var nativeKeys = overArg(Object.keys, Object);

/* Built-in method references that are verified to be native. */
var DataView = getNative(root$1, 'DataView'),
  Map = getNative(root$1, 'Map'),
  Promise$1 = getNative(root$1, 'Promise'),
  Set = getNative(root$1, 'Set'),
  WeakMap = getNative(root$1, 'WeakMap');

/** Used to detect maps, sets, and weakmaps. */
var dataViewCtorString = toSource(DataView),
  mapCtorString = toSource(Map),
  promiseCtorString = toSource(Promise$1),
  setCtorString = toSource(Set),
  weakMapCtorString = toSource(WeakMap);

/**
 * Creates an array of the enumerable property names of the array-like `value`.
 *
 * @private
 * @param {*} value The value to query.
 * @param {boolean} inherited Specify returning inherited property names.
 * @returns {Array} Returns the array of property names.
 */
function arrayLikeKeys(value, inherited) {
  // Safari 8.1 makes `arguments.callee` enumerable in strict mode.
  // Safari 9 makes `arguments.length` enumerable in strict mode.
  var result = isArray(value) || isArguments(value) ? baseTimes(value.length, String) : [];
  var length = result.length,
    skipIndexes = !!length;
  for (var key in value) {
    if ((inherited || hasOwnProperty.call(value, key)) && !(skipIndexes && (key == 'length' || isIndex(key, length)))) {
      result.push(key);
    }
  }
  return result;
}

/**
 * The base implementation of `getTag`.
 *
 * @private
 * @param {*} value The value to query.
 * @returns {string} Returns the `toStringTag`.
 */
function baseGetTag(value) {
  return objectToString$1.call(value);
}

/**
 * The base implementation of `_.isNative` without bad shim checks.
 *
 * @private
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is a native function,
 *  else `false`.
 */
function baseIsNative(value) {
  if (!isObject$1(value) || isMasked(value)) {
    return false;
  }
  var pattern = isFunction(value) || isHostObject(value) ? reIsNative : reIsHostCtor;
  return pattern.test(toSource(value));
}

/**
 * The base implementation of `_.keys` which doesn't treat sparse arrays as dense.
 *
 * @private
 * @param {Object} object The object to query.
 * @returns {Array} Returns the array of property names.
 */
function baseKeys(object) {
  if (!isPrototype(object)) {
    return nativeKeys(object);
  }
  var result = [];
  for (var key in Object(object)) {
    if (hasOwnProperty.call(object, key) && key != 'constructor') {
      result.push(key);
    }
  }
  return result;
}

/**
 * Creates a `_.toPairs` or `_.toPairsIn` function.
 *
 * @private
 * @param {Function} keysFunc The function to get the keys of a given object.
 * @returns {Function} Returns the new pairs function.
 */
function createToPairs(keysFunc) {
  return function (object) {
    var tag = getTag(object);
    if (tag == mapTag) {
      return mapToArray(object);
    }
    if (tag == setTag) {
      return setToPairs(object);
    }
    return baseToPairs(object, keysFunc(object));
  };
}

/**
 * Gets the native function at `key` of `object`.
 *
 * @private
 * @param {Object} object The object to query.
 * @param {string} key The key of the method to get.
 * @returns {*} Returns the function if it's native, else `undefined`.
 */
function getNative(object, key) {
  var value = getValue(object, key);
  return baseIsNative(value) ? value : undefined;
}

/**
 * Gets the `toStringTag` of `value`.
 *
 * @private
 * @param {*} value The value to query.
 * @returns {string} Returns the `toStringTag`.
 */
var getTag = baseGetTag;

// Fallback for data views, maps, sets, and weak maps in IE 11,
// for data views in Edge < 14, and promises in Node.js.
if (DataView && getTag(new DataView(new ArrayBuffer(1))) != dataViewTag || Map && getTag(new Map()) != mapTag || Promise$1 && getTag(Promise$1.resolve()) != promiseTag || Set && getTag(new Set()) != setTag || WeakMap && getTag(new WeakMap()) != weakMapTag) {
  getTag = function (value) {
    var result = objectToString$1.call(value),
      Ctor = result == objectTag ? value.constructor : undefined,
      ctorString = Ctor ? toSource(Ctor) : undefined;
    if (ctorString) {
      switch (ctorString) {
        case dataViewCtorString:
          return dataViewTag;
        case mapCtorString:
          return mapTag;
        case promiseCtorString:
          return promiseTag;
        case setCtorString:
          return setTag;
        case weakMapCtorString:
          return weakMapTag;
      }
    }
    return result;
  };
}

/**
 * Checks if `value` is a valid array-like index.
 *
 * @private
 * @param {*} value The value to check.
 * @param {number} [length=MAX_SAFE_INTEGER] The upper bounds of a valid index.
 * @returns {boolean} Returns `true` if `value` is a valid index, else `false`.
 */
function isIndex(value, length) {
  length = length == null ? MAX_SAFE_INTEGER : length;
  return !!length && (typeof value == 'number' || reIsUint.test(value)) && value > -1 && value % 1 == 0 && value < length;
}

/**
 * Checks if `func` has its source masked.
 *
 * @private
 * @param {Function} func The function to check.
 * @returns {boolean} Returns `true` if `func` is masked, else `false`.
 */
function isMasked(func) {
  return !!maskSrcKey && maskSrcKey in func;
}

/**
 * Checks if `value` is likely a prototype object.
 *
 * @private
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is a prototype, else `false`.
 */
function isPrototype(value) {
  var Ctor = value && value.constructor,
    proto = typeof Ctor == 'function' && Ctor.prototype || objectProto$1;
  return value === proto;
}

/**
 * Converts `func` to its source code.
 *
 * @private
 * @param {Function} func The function to process.
 * @returns {string} Returns the source code.
 */
function toSource(func) {
  if (func != null) {
    try {
      return funcToString.call(func);
    } catch (e) {}
    try {
      return func + '';
    } catch (e) {}
  }
  return '';
}

/**
 * Checks if `value` is likely an `arguments` object.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is an `arguments` object,
 *  else `false`.
 * @example
 *
 * _.isArguments(function() { return arguments; }());
 * // => true
 *
 * _.isArguments([1, 2, 3]);
 * // => false
 */
function isArguments(value) {
  // Safari 8.1 makes `arguments.callee` enumerable in strict mode.
  return isArrayLikeObject(value) && hasOwnProperty.call(value, 'callee') && (!propertyIsEnumerable.call(value, 'callee') || objectToString$1.call(value) == argsTag);
}

/**
 * Checks if `value` is classified as an `Array` object.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is an array, else `false`.
 * @example
 *
 * _.isArray([1, 2, 3]);
 * // => true
 *
 * _.isArray(document.body.children);
 * // => false
 *
 * _.isArray('abc');
 * // => false
 *
 * _.isArray(_.noop);
 * // => false
 */
var isArray = Array.isArray;

/**
 * Checks if `value` is array-like. A value is considered array-like if it's
 * not a function and has a `value.length` that's an integer greater than or
 * equal to `0` and less than or equal to `Number.MAX_SAFE_INTEGER`.
 *
 * @static
 * @memberOf _
 * @since 4.0.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is array-like, else `false`.
 * @example
 *
 * _.isArrayLike([1, 2, 3]);
 * // => true
 *
 * _.isArrayLike(document.body.children);
 * // => true
 *
 * _.isArrayLike('abc');
 * // => true
 *
 * _.isArrayLike(_.noop);
 * // => false
 */
function isArrayLike(value) {
  return value != null && isLength(value.length) && !isFunction(value);
}

/**
 * This method is like `_.isArrayLike` except that it also checks if `value`
 * is an object.
 *
 * @static
 * @memberOf _
 * @since 4.0.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is an array-like object,
 *  else `false`.
 * @example
 *
 * _.isArrayLikeObject([1, 2, 3]);
 * // => true
 *
 * _.isArrayLikeObject(document.body.children);
 * // => true
 *
 * _.isArrayLikeObject('abc');
 * // => false
 *
 * _.isArrayLikeObject(_.noop);
 * // => false
 */
function isArrayLikeObject(value) {
  return isObjectLike$1(value) && isArrayLike(value);
}

/**
 * Checks if `value` is classified as a `Function` object.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is a function, else `false`.
 * @example
 *
 * _.isFunction(_);
 * // => true
 *
 * _.isFunction(/abc/);
 * // => false
 */
function isFunction(value) {
  // The use of `Object#toString` avoids issues with the `typeof` operator
  // in Safari 8-9 which returns 'object' for typed array and other constructors.
  var tag = isObject$1(value) ? objectToString$1.call(value) : '';
  return tag == funcTag || tag == genTag;
}

/**
 * Checks if `value` is a valid array-like length.
 *
 * **Note:** This method is loosely based on
 * [`ToLength`](http://ecma-international.org/ecma-262/7.0/#sec-tolength).
 *
 * @static
 * @memberOf _
 * @since 4.0.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is a valid length, else `false`.
 * @example
 *
 * _.isLength(3);
 * // => true
 *
 * _.isLength(Number.MIN_VALUE);
 * // => false
 *
 * _.isLength(Infinity);
 * // => false
 *
 * _.isLength('3');
 * // => false
 */
function isLength(value) {
  return typeof value == 'number' && value > -1 && value % 1 == 0 && value <= MAX_SAFE_INTEGER;
}

/**
 * Checks if `value` is the
 * [language type](http://www.ecma-international.org/ecma-262/7.0/#sec-ecmascript-language-types)
 * of `Object`. (e.g. arrays, functions, objects, regexes, `new Number(0)`, and `new String('')`)
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is an object, else `false`.
 * @example
 *
 * _.isObject({});
 * // => true
 *
 * _.isObject([1, 2, 3]);
 * // => true
 *
 * _.isObject(_.noop);
 * // => true
 *
 * _.isObject(null);
 * // => false
 */
function isObject$1(value) {
  var type = typeof value;
  return !!value && (type == 'object' || type == 'function');
}

/**
 * Checks if `value` is object-like. A value is object-like if it's not `null`
 * and has a `typeof` result of "object".
 *
 * @static
 * @memberOf _
 * @since 4.0.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is object-like, else `false`.
 * @example
 *
 * _.isObjectLike({});
 * // => true
 *
 * _.isObjectLike([1, 2, 3]);
 * // => true
 *
 * _.isObjectLike(_.noop);
 * // => false
 *
 * _.isObjectLike(null);
 * // => false
 */
function isObjectLike$1(value) {
  return !!value && typeof value == 'object';
}

/**
 * Creates an array of the own enumerable property names of `object`.
 *
 * **Note:** Non-object values are coerced to objects. See the
 * [ES spec](http://ecma-international.org/ecma-262/7.0/#sec-object.keys)
 * for more details.
 *
 * @static
 * @since 0.1.0
 * @memberOf _
 * @category Object
 * @param {Object} object The object to query.
 * @returns {Array} Returns the array of property names.
 * @example
 *
 * function Foo() {
 *   this.a = 1;
 *   this.b = 2;
 * }
 *
 * Foo.prototype.c = 3;
 *
 * _.keys(new Foo);
 * // => ['a', 'b'] (iteration order is not guaranteed)
 *
 * _.keys('hi');
 * // => ['0', '1']
 */
function keys(object) {
  return isArrayLike(object) ? arrayLikeKeys(object) : baseKeys(object);
}

/**
 * Creates an array of own enumerable string keyed-value pairs for `object`
 * which can be consumed by `_.fromPairs`. If `object` is a map or set, its
 * entries are returned.
 *
 * @static
 * @memberOf _
 * @since 4.0.0
 * @alias entries
 * @category Object
 * @param {Object} object The object to query.
 * @returns {Array} Returns the key-value pairs.
 * @example
 *
 * function Foo() {
 *   this.a = 1;
 *   this.b = 2;
 * }
 *
 * Foo.prototype.c = 3;
 *
 * _.toPairs(new Foo);
 * // => [['a', 1], ['b', 2]] (iteration order is not guaranteed)
 */
var toPairs = createToPairs(keys);
var lodash_topairs = toPairs;
var PopoverContent = function (_a) {
  var className = _a.className,
    metadata = _a.metadata,
    children = _a.children,
    popoverRef = _a.popoverRef,
    style = _a.style;
  return /*#__PURE__*/React.createElement("div", __assign({
    ref: popoverRef,
    className: className,
    style: style
  }, parseMetadata(suffixMetadata(metadata, 'list'))), children);
};
PopoverContent.displayName = 'Content';
var PopoverTrigger = function (_a) {
  var children = _a.children,
    triggerRef = _a.triggerRef,
    passedProps = __rest(_a, ["children", "triggerRef"]);
  var customRef = useForkRef([triggerRef, null]);
  // Backwards compatibility: custom method used to render a bespoke trigger
  if (typeof children === 'function') {
    var editedProps = __assign(__assign({}, passedProps), {
      ref: customRef
    });
    return children(editedProps);
  }
  if (children) {
    var child = React.Children.only(children);
    return /*#__PURE__*/React.cloneElement(child, __assign({
      ref: customRef
    }, passedProps));
  }
  return null;
};
PopoverTrigger.displayName = 'Trigger';
var useListVisibilityChangeHandler = function (onListVisibilityChange, visible) {
  var initialRender = useRef(true);
  useEffect(function () {
    if (initialRender.current) {
      initialRender.current = false;
    } else {
      onListVisibilityChange(visible);
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [visible]);
};
var Popover = /*#__PURE__*/React.forwardRef(function (_a, ref) {
  var className = _a.className,
    _b = _a.variant,
    variant = _b === void 0 ? 'inline' : _b,
    metadata = _a.metadata,
    _c = _a.onListVisibilityChange,
    onListVisibilityChange = _c === void 0 ? noop : _c,
    _d = _a.isLoading,
    isLoading = _d === void 0 ? false : _d,
    children = _a.children,
    _e = _a.isVisible,
    isVisible = _e === void 0 ? false : _e;
  var selectTriggerRef = useRef(null);
  var _f = useState(isVisible),
    visible = _f[0],
    setVisibility = _f[1];
  var _g = useState(),
    triggerWidth = _g[0],
    setTriggerWidth = _g[1];
  useEffect(function () {
    setVisibility(isVisible);
  }, [isVisible]);
  var _h = useAutoPosition(),
    floatTriggerRef = _h.triggerRef,
    floatRef = _h.floatRef,
    floatStyles = _h.styles;
  var triggerRef = useForkRef([floatTriggerRef, function (elem) {
    if (elem) setTriggerWidth(getComputedStyle(elem).width);
  }]);
  var onClickOutsideRef = useClickOutside(function () {
    return visible && setVisibility(false);
  });
  var finalRef = useForkRef([ref, selectTriggerRef, onClickOutsideRef]);
  useListVisibilityChangeHandler(onListVisibilityChange, visible);
  var handleTriggerClick = function () {
    setVisibility(function (currentVisibility) {
      return !currentVisibility;
    });
  };
  return /*#__PURE__*/React.createElement("div", __assign({
    className: className,
    ref: finalRef
  }, parseMetadata(metadata)), React.Children.map(children, function (listItem) {
    if (! /*#__PURE__*/React.isValidElement(listItem)) {
      return null;
    }
    switch (listItem.type.displayName) {
      case PopoverTrigger.displayName:
        return /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.cloneElement(listItem, {
          selectRef: selectTriggerRef,
          triggerRef: triggerRef,
          onClick: handleTriggerClick,
          visible: visible,
          variant: variant
        }));
      case PopoverContent.displayName:
        return /*#__PURE__*/React.createElement(React.Fragment, null, visible && /*#__PURE__*/React.cloneElement(listItem, {
          popoverRef: floatRef,
          selectTriggerRef: selectTriggerRef,
          style: __assign(__assign(__assign({}, floatStyles), variant === 'full-width' ? {
            width: triggerWidth
          } : {}), isLoading ? {
            minWidth: triggerWidth
          } : {})
        }));
      default:
        return null;
    }
  }));
});
Popover.Trigger = PopoverTrigger;
Popover.Content = PopoverContent;
var searchSubstring = function (string, substring) {
  if (string === void 0) {
    string = '';
  }
  if (substring === void 0) {
    substring = '';
  }
  return string.toLowerCase().indexOf(substring.toLowerCase());
};
function getSelectedValues(_a) {
  var multiple = _a.multiple,
    option = _a.option,
    selectedOptions = _a.selectedOptions,
    selectedOptionsWithValuesOnly = _a.selectedOptionsWithValuesOnly;
  if (!multiple) {
    return [option];
  }
  if (selectedOptionsWithValuesOnly.includes(option.value)) {
    return selectedOptions.filter(function (selectedOption) {
      return selectedOption.value !== option.value;
    });
  }
  return __spreadArray(__spreadArray([], selectedOptions, true), [option], false);
}
/**
 * Returns options that are pre-selected and disabled.
 * @param allOptions
 * @param preSelectedOptions
 */
function getFixedOptions(allOptions, preSelectedOptions) {
  var allDisabled = allOptions.filter(function (option) {
    return option.disabled;
  });
  return allDisabled.filter(function (disabledOption) {
    return preSelectedOptions.some(function (option) {
      return option.value === disabledOption.value;
    });
  });
}
/**
 * Returns a list of option that are fixed and are enabled
 * @param options
 * @param preSelectedOptions
 */
function getAllSelectableOptions(options, preSelectedOptions) {
  var allFixed = getFixedOptions(options, preSelectedOptions);
  var allEnabled = options.filter(function (option) {
    return !option.disabled;
  });
  return __spreadArray(__spreadArray([], allFixed, true), allEnabled, true);
}
function applyProps(children, cb) {
  return React.Children.map(children, function (c) {
    if (! /*#__PURE__*/React.isValidElement(c)) {
      return c;
    }
    var child = c;
    var grandChildren = child.props.children;
    if (grandChildren) {
      var cloned = /*#__PURE__*/React.cloneElement(child, {
        children: applyProps(grandChildren, cb)
      });
      return cb(cloned);
    }
    return cb(child);
  });
}
function extractProps(children, predicate) {
  var props = [];
  React.Children.forEach(children, function (child) {
    if (! /*#__PURE__*/React.isValidElement(child)) {
      return;
    }
    var grandChildren = child.props.children;
    if (grandChildren) {
      var grandProps = extractProps(grandChildren, predicate);
      props = __spreadArray(__spreadArray([], props, true), grandProps, true);
    }
    if (predicate(child)) {
      props.push(child.props);
    }
  });
  return props;
}

/* eslint-disable no-console */
var SelectContext = /*#__PURE__*/createContext({
  multiple: false,
  isClearable: false,
  isLoading: false,
  isVisible: false,
  setIsVisible: console.error,
  searchInputValue: '',
  setSearchInputValue: console.error,
  options: [],
  setOptions: console.error,
  selectedOptions: [],
  filteredOptions: [],
  selectedOptionsWithValuesOnly: [],
  shouldShowSearchInput: false,
  selectAllIsSelected: false,
  shouldShowNoResultsLabel: false,
  isFullWidthVariant: false,
  optionType: 'single-line',
  onSelect: console.error,
  onSearchChange: console.error
});
function SelectProvider(_a) {
  var children = _a.children,
    _b = _a.selectedOptions,
    selectedOptions = _b === void 0 ? [] : _b,
    _c = _a.minOptionsToDisplaySearch,
    minOptionsToDisplaySearch = _c === void 0 ? 8 : _c,
    _d = _a.variant,
    variant = _d === void 0 ? 'inline' : _d,
    _e = _a.optionType,
    optionType = _e === void 0 ? 'single-line' : _e,
    _f = _a.onSelect,
    onSelect = _f === void 0 ? noop : _f,
    _g = _a.multiple,
    multiple = _g === void 0 ? false : _g,
    _h = _a.isClearable,
    isClearable = _h === void 0 ? false : _h,
    _j = _a.isLoading,
    isLoading = _j === void 0 ? false : _j,
    _k = _a.searchInitialValue,
    searchInitialValue = _k === void 0 ? '' : _k,
    _l = _a.onSearchChange,
    onSearchChange = _l === void 0 ? noop : _l;
  var _m = useState(false),
    isVisible = _m[0],
    setIsVisible = _m[1];
  var _o = useState(searchInitialValue),
    searchInputValue = _o[0],
    setSearchInputValue = _o[1];
  var _p = useState([]),
    options = _p[0],
    setOptions = _p[1];
  var selectedOptionsWithValuesOnly = useMemo(function () {
    return (selectedOptions === null || selectedOptions === void 0 ? void 0 : selectedOptions.map(function (selectedOption) {
      return selectedOption.value;
    })) || [];
  }, [selectedOptions]);
  var filteredOptions = useMemo(function () {
    return options.filter(function (option) {
      return searchSubstring(option === null || option === void 0 ? void 0 : option.label, searchInputValue) !== -1;
    });
  }, [options, searchInputValue]);
  useEffect(function () {
    onSearchChange(searchInputValue);
  }, [onSearchChange, searchInputValue]);
  var value = useMemo(function () {
    return {
      optionType: optionType,
      isVisible: isVisible,
      setIsVisible: setIsVisible,
      multiple: multiple,
      isClearable: isClearable,
      isLoading: isLoading,
      searchInputValue: searchInputValue,
      setSearchInputValue: setSearchInputValue,
      options: options,
      setOptions: setOptions,
      selectedOptions: selectedOptions,
      filteredOptions: filteredOptions,
      selectedOptionsWithValuesOnly: selectedOptionsWithValuesOnly,
      shouldShowSearchInput: options.length >= minOptionsToDisplaySearch,
      selectAllIsSelected: options.length === selectedOptionsWithValuesOnly.length,
      shouldShowNoResultsLabel: searchInputValue !== '' && filteredOptions.length === 0,
      isFullWidthVariant: variant === 'full-width',
      onSelect: onSelect,
      onSearchChange: onSearchChange
    };
  }, [filteredOptions, isClearable, isLoading, isVisible, minOptionsToDisplaySearch, multiple, onSearchChange, onSelect, optionType, options, searchInputValue, selectedOptions, selectedOptionsWithValuesOnly, variant]);
  return /*#__PURE__*/React.createElement(SelectContext.Provider, {
    value: value
  }, children);
}
var styles$f = {
  "labelWrapper": "Option-module__52XUwPUP__v9-4-8",
  "option": "Option-module__j6IMCrhL__v9-4-8",
  "optionIcon": "Option-module__7WkwcyVK__v9-4-8",
  "optionCheckbox": "Option-module__BWRRvu90__v9-4-8",
  "multiLineOption": "Option-module__6bVt-vn9__v9-4-8",
  "selectedText": "Option-module__keOUg9Dt__v9-4-8",
  "disabled": "Option-module__ptIUDrHE__v9-4-8",
  "multiLineIcon": "Option-module__ZFlI5xTP__v9-4-8",
  "multiLineCheckbox": "Option-module__7cQCCdjv__v9-4-8"
};
var SingleLineOptionLabel = function (_a) {
  var highlightedText = _a.highlightedText,
    text = _a.option.label,
    _b = _a.selected,
    selected = _b === void 0 ? false : _b,
    _c = _a.disabled,
    disabled = _c === void 0 ? false : _c;
  var startIndex = searchSubstring(text, highlightedText);
  var endIndex = startIndex + highlightedText.length;
  return /*#__PURE__*/React.createElement(Typography, {
    component: "span",
    token: selected ? 'body-strong-base' : 'body-base',
    color: disabled ? 'text-disabled' : 'text-default'
  }, startIndex === -1 ? text : /*#__PURE__*/React.createElement(React.Fragment, null, text.substring(0, startIndex), /*#__PURE__*/React.createElement(Typography, {
    component: "b",
    token: "body-strong-base"
  }, text.substring(startIndex, endIndex)), text.substring(endIndex)));
};
var styles$e = {
  "label": "MultiLineOptionLabel-module__exBQV2M-__v9-4-8",
  "subLabel": "MultiLineOptionLabel-module__Wg-fF8xB__v9-4-8",
  "truncated": "MultiLineOptionLabel-module__lBD4-eDy__v9-4-8",
  "disabled": "MultiLineOptionLabel-module__D5Vz64E-__v9-4-8"
};
function MultiLineOptionLabel(_a) {
  var _b;
  var option = _a.option,
    highlightedText = _a.highlightedText,
    selected = _a.selected,
    disabled = _a.disabled,
    _c = _a.truncateText,
    truncateText = _c === void 0 ? false : _c;
  return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("span", {
    className: styles$e.label
  }, /*#__PURE__*/React.createElement(SingleLineOptionLabel, {
    highlightedText: highlightedText,
    option: option,
    selected: selected,
    disabled: disabled
  })), /*#__PURE__*/React.createElement(Typography, {
    className: classnames(styles$e.subLabel, (_b = {}, _b[styles$e.disabled] = option.disabled, _b[styles$e.truncated] = truncateText, _b)),
    component: "span",
    token: "body-base",
    title: option.subLabel
  }, option.subLabel));
}
var styles$d = {
  "wrapper": "AvatarOptionLabel-module__vUHDd8Up__v9-4-8",
  "avatar": "AvatarOptionLabel-module__5rUwf3mv__v9-4-8",
  "disabledAvatar": "AvatarOptionLabel-module__u6ImUvDM__v9-4-8",
  "labelsWrapper": "AvatarOptionLabel-module__P158DTjO__v9-4-8"
};
function AvatarOptionLabel(_a) {
  var _b;
  var option = _a.option,
    highlightedText = _a.highlightedText,
    selected = _a.selected,
    disabled = _a.disabled;
  return /*#__PURE__*/React.createElement("div", {
    className: styles$d.wrapper
  }, /*#__PURE__*/React.createElement(Avatar, {
    src: option.avatar,
    name: option.label,
    containerClassname: classnames(styles$d.avatar, (_b = {}, _b[styles$d.disabledAvatar] = option.disabled, _b)),
    imageLoading: option === null || option === void 0 ? void 0 : option.imageLoading
  }), /*#__PURE__*/React.createElement("div", {
    className: styles$d.labelsWrapper
  }, /*#__PURE__*/React.createElement(MultiLineOptionLabel, {
    highlightedText: highlightedText,
    option: option,
    selected: selected,
    disabled: disabled,
    truncateText: true
  })));
}
var styles$c = {
  "colorIcon": "ColoredOptionLabel-module__EsIwwswX__v9-4-8",
  "coloredOption": "ColoredOptionLabel-module__8qVD9vWq__v9-4-8"
};
function ColoredOptionLabel(_a) {
  var _b;
  var option = _a.option,
    highlightedText = _a.highlightedText,
    selected = _a.selected,
    disabled = _a.disabled;
  return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("span", {
    className: classnames(styles$c.colorIcon, (_b = {}, _b[styles$c.coloredOption] = !option.disabled, _b)),
    style: !option.disabled ? {
      '--colored-option': "" + option.color
    } : undefined
  }, /*#__PURE__*/React.createElement(Icon, {
    name: "circle",
    type: "solid",
    size: 12
  })), /*#__PURE__*/React.createElement(SingleLineOptionLabel, {
    highlightedText: highlightedText,
    option: option,
    selected: selected,
    disabled: disabled
  }));
}
var optionLabelComponents = {
  'single-line': SingleLineOptionLabel,
  'multi-line': MultiLineOptionLabel,
  colored: ColoredOptionLabel,
  avatar: AvatarOptionLabel
};
var Option = function (_a) {
  var _b, _c, _d;
  var disabled = _a.disabled,
    highlightedText = _a.highlightedText,
    onSelect = _a.onSelect,
    onEscape = _a.onEscape,
    option = _a.option,
    optionStyle = _a.optionStyle,
    optionType = _a.optionType,
    selected = _a.selected,
    className = _a.className;
  var handleKeyDown = function (e) {
    if ([' ', 'Enter'].some(function (k) {
      return k === e.key;
    })) {
      e.preventDefault();
      onSelect(option);
    } else if (e.key === 'Escape') {
      onEscape();
    }
  };
  return /*#__PURE__*/React.createElement("li", __assign({
    "aria-selected": selected,
    className: classnames(styles$f.option, (_b = {}, _b[styles$f.selectedText] = selected && optionStyle === 'checkmark', _b[styles$f.multiLineOption] = optionType === 'multi-line', _b[styles$f.disabled] = disabled, _b), className),
    onClick: !disabled ? function () {
      return onSelect(option);
    } : noop,
    onKeyDown: !disabled ? handleKeyDown : noop,
    role: "option",
    title: option.label,
    tabIndex: 0
  }, parseMetadata(option.metadata)), optionStyle === 'checkbox' ? /*#__PURE__*/React.createElement(Checkbox, {
    className: classnames(styles$f.optionCheckbox, (_c = {}, _c[styles$f.multiLineCheckbox] = optionType === 'multi-line', _c)),
    checked: selected,
    readOnly: true,
    disabled: disabled,
    tabIndex: -1
  }) : /*#__PURE__*/React.createElement(Icon, {
    className: classnames(styles$f.optionIcon, (_d = {}, _d[styles$f.multiLineIcon] = optionType === 'multi-line', _d)),
    name: "check",
    size: 14,
    type: "solid"
  }), /*#__PURE__*/React.createElement("div", {
    className: styles$f.labelWrapper
  }, /*#__PURE__*/React.createElement(optionLabelComponents[optionType], {
    option: option,
    highlightedText: highlightedText,
    selected: optionStyle !== 'checkbox' ? selected : false,
    disabled: disabled
  })));
};
Option.displayName = 'Option';
var styles$b = {
  "optionGroup": "OptionGroup-module__PoRdDGLe__v9-4-8",
  "optionGroupName": "OptionGroup-module__AamEvMnW__v9-4-8"
};
var OptionGroup = function (_a) {
  var name = _a.name,
    children = _a.children;
  return /*#__PURE__*/React.createElement("li", {
    className: styles$b.optionGroup
  }, /*#__PURE__*/React.createElement(Typography, {
    className: styles$b.optionGroupName,
    token: "heading-small",
    component: "div"
  }, name), /*#__PURE__*/React.createElement("ul", null, children));
};
var styles$a = {
  "selectAllOption": "SelectAllOption-module__SlAwog5k__v9-4-8"
};
var SelectAllOption = function (_a) {
  var _b = _a.label,
    label = _b === void 0 ? 'Select all' : _b,
    passedProps = __rest(_a, ["label"]);
  var _c = passedProps,
    isSelected = _c.isSelected,
    onSelect = _c.onSelect,
    _d = _c.onEscape,
    onEscape = _d === void 0 ? noop : _d;
  var selectAllOption = useMemo(function () {
    return {
      value: Date.now(),
      label: label
    };
  }, [label]);
  return /*#__PURE__*/React.createElement(Option, {
    highlightedText: "",
    onSelect: function () {
      return onSelect(!isSelected);
    },
    option: selectAllOption,
    optionStyle: "checkbox",
    optionType: "single-line",
    selected: isSelected,
    className: styles$a.selectAllOption,
    onEscape: onEscape
  });
};
var ListBox = function (_a) {
  var children = _a.children,
    _b = _a.searchable,
    searchable = _b === void 0 ? true : _b,
    _c = _a.highlightedText,
    highlightedText = _c === void 0 ? '' : _c,
    _d = _a.multiple,
    multiple = _d === void 0 ? false : _d,
    _e = _a.searchInputValue,
    searchInputValue = _e === void 0 ? '' : _e,
    selectAllIsSelected = _a.selectAllIsSelected,
    selectedOptionsWithValuesOnly = _a.selectedOptionsWithValuesOnly,
    listRef = _a.listRef,
    onEscape = _a.onEscape,
    handleSelect = _a.handleSelect,
    handleSelectAll = _a.handleSelectAll,
    _f = _a.optionType,
    optionType = _f === void 0 ? 'single-line' : _f;
  var subcomponents = applyProps(children, function (child) {
    var isOption = child.type.displayName === Option.displayName;
    var isSelectAll = child.type === SelectAllOption;
    var isOptionGroup = child.type === OptionGroup;
    if (isOption) {
      var value = child.props.value;
      return /*#__PURE__*/React.cloneElement(child, {
        onEscape: onEscape,
        onSelect: handleSelect,
        selected: selectedOptionsWithValuesOnly.includes(value),
        optionStyle: multiple ? 'checkbox' : 'checkmark',
        optionType: optionType !== null && optionType !== void 0 ? optionType : 'single-line',
        highlightedText: searchable ? searchInputValue : highlightedText
      });
    }
    if (isSelectAll) {
      if ((searchInputValue === null || searchInputValue === void 0 ? void 0 : searchInputValue.length) === 0 && multiple) {
        return /*#__PURE__*/React.cloneElement(child, {
          onEscape: onEscape,
          onSelect: handleSelectAll,
          isSelected: selectAllIsSelected
        });
      }
      return null;
    }
    if (isOptionGroup) {
      return child;
    }
    return null;
  });
  return /*#__PURE__*/React.createElement("ul", {
    role: "listbox",
    ref: listRef
  }, subcomponents);
};
ListBox.displayName = 'ListBox';
ListBox.SelectAllOption = SelectAllOption;
ListBox.OptionGroup = OptionGroup;
ListBox.Option = Option;
var SelectListBoxOption = function (_a) {
  var children = _a.children;
  _a.className;
  var props = __rest(_a, ["children", "className"]);
  var childrenArray = React.Children.toArray(children);
  if (childrenArray.length === 0) {
    return null;
  }
  var labelElement = childrenArray[0];
  if (typeof labelElement !== 'string') {
    return null;
  }
  var color = props.color,
    disabled = props.disabled,
    group = props.group,
    subLabel = props.subLabel,
    imageLoading = props.imageLoading,
    metadata = props.metadata,
    value = props.value,
    avatar = props.avatar,
    restProps = __rest(props, ["color", "disabled", "group", "subLabel", "imageLoading", "metadata", "value", "avatar"]);
  return /*#__PURE__*/React.createElement(ListBox.Option, __assign({}, restProps, {
    disabled: disabled,
    option: {
      color: color,
      disabled: disabled,
      group: group,
      subLabel: subLabel,
      imageLoading: imageLoading,
      metadata: metadata,
      value: value,
      avatar: avatar,
      label: labelElement
    }
  }));
};
SelectListBoxOption.displayName = 'Option';
var SelectListBox = function (_a) {
  var children = _a.children,
    props = __rest(_a, ["children"]);
  var _b = useContext(SelectContext),
    multiple = _b.multiple,
    selectAllIsSelected = _b.selectAllIsSelected,
    searchInputValue = _b.searchInputValue,
    filteredOptions = _b.filteredOptions,
    selectedOptionsWithValuesOnly = _b.selectedOptionsWithValuesOnly;
  var filteredOptionChildren = applyProps(children, function (child) {
    var _a, _b;
    var isOption = child.type === ListBox.Option || child.type.displayName === ListBox.Option.displayName;
    var isOptionGroup = child.type === OptionGroup;
    if (isOption) {
      var childProps = child.props;
      var value_1 = (_b = (_a = childProps.option) === null || _a === void 0 ? void 0 : _a.value) !== null && _b !== void 0 ? _b : childProps === null || childProps === void 0 ? void 0 : childProps.value;
      var shouldBeInResultSet = filteredOptions.find(function (o) {
        return o.value === value_1;
      });
      if (!shouldBeInResultSet) {
        return;
      }
    }
    if (isOptionGroup) {
      var childProps = child.props;
      var groupChildren = React.Children.toArray(childProps.children);
      if (groupChildren.length !== 0) {
        return /*#__PURE__*/React.cloneElement(child);
      }
      return null;
    }
    return /*#__PURE__*/React.cloneElement(child);
  });
  var optionGroupsProps = extractProps(filteredOptionChildren, function (child) {
    return child.type === OptionGroup;
  });
  var filteredChildren = applyProps(filteredOptionChildren, function (child) {
    var isOptionGroup = child.type === OptionGroup;
    if (isOptionGroup && optionGroupsProps.length === 1) {
      var childProps = child.props;
      return childProps.children;
    }
    return /*#__PURE__*/React.cloneElement(child);
  });
  return /*#__PURE__*/React.createElement(ListBox, __assign({}, props, {
    searchInputValue: searchInputValue,
    selectAllIsSelected: selectAllIsSelected,
    selectedOptionsWithValuesOnly: selectedOptionsWithValuesOnly !== null && selectedOptionsWithValuesOnly !== void 0 ? selectedOptionsWithValuesOnly : [],
    multiple: multiple
  }), filteredChildren);
};
SelectListBox.displayName = 'ListBox';
SelectListBox.Option = SelectListBoxOption;
SelectListBox.OptionGroup = ListBox.OptionGroup;
SelectListBox.SelectAllOption = ListBox.SelectAllOption;
var SelectPopover = /*#__PURE__*/React.forwardRef(function (_a, ref) {
  var className = _a.className,
    _b = _a.variant,
    variant = _b === void 0 ? 'inline' : _b,
    metadata = _a.metadata,
    _c = _a.onListVisibilityChange,
    onListVisibilityChange = _c === void 0 ? noop : _c,
    children = _a.children;
  var _d = useContext(SelectContext),
    isLoading = _d.isLoading,
    isVisible = _d.isVisible,
    setIsVisible = _d.setIsVisible,
    setOptions = _d.setOptions;
  useEffect(function () {
    var basicOptionProps = extractProps(children, function (child) {
      return child.type === SelectListBox.Option;
    });
    setOptions(basicOptionProps.map(function (o) {
      return {
        label: o.children,
        value: o.value,
        disabled: o.disabled
      };
    }));
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [children]);
  return /*#__PURE__*/React.createElement(Popover, {
    isVisible: isVisible,
    className: className,
    variant: variant,
    metadata: metadata,
    onListVisibilityChange: function (toggle) {
      onListVisibilityChange(toggle);
      setIsVisible(toggle);
    },
    isLoading: isLoading,
    ref: ref
  }, children);
});
var styles$9 = {
  "label": "SelectContent-module__nq3nhBIM__v9-4-8",
  "subLabel": "SelectContent-module__UoTda34P__v9-4-8",
  "truncated": "SelectContent-module__xyXWxggf__v9-4-8",
  "labelWrapper": "SelectContent-module__YS-9otbQ__v9-4-8",
  "list": "SelectContent-module__msac0G63__v9-4-8",
  "option": "SelectContent-module__qN2zDEBs__v9-4-8",
  "optionIcon": "SelectContent-module__Eb8IBfh-__v9-4-8",
  "optionCheckbox": "SelectContent-module__N6wSkZS9__v9-4-8",
  "multiLineOption": "SelectContent-module__iIKSFP4E__v9-4-8",
  "selectedText": "SelectContent-module__xS2JWsqf__v9-4-8",
  "disabled": "SelectContent-module__6CPx7vyH__v9-4-8",
  "multiLineIcon": "SelectContent-module__Wsxrepf0__v9-4-8",
  "multiLineCheckbox": "SelectContent-module__5LyxFpTd__v9-4-8",
  "listboxOverflowScroll": "SelectContent-module__GYyQ1Tjn__v9-4-8",
  "loadingContainer": "SelectContent-module__-TpB4Gjn__v9-4-8",
  "multiLineList": "SelectContent-module__G6-af0pv__v9-4-8",
  "fullWidth": "SelectContent-module__6Hi40cWk__v9-4-8"
};
var styles$8 = {
  "heading-xsmall": "SearchInput-module__epSLVUB0__v9-4-8",
  "heading-small": "SearchInput-module__9tVerypd__v9-4-8",
  "heading-medium": "SearchInput-module__cmnQkxN0__v9-4-8",
  "heading-large": "SearchInput-module__GYEIJyJO__v9-4-8",
  "heading-xlarge": "SearchInput-module__PDP-kTiP__v9-4-8",
  "heading-2xl": "SearchInput-module__Q86O-MjL__v9-4-8",
  "body-small": "SearchInput-module__M--qxOXL__v9-4-8",
  "body-base": "SearchInput-module__o5dBA4e-__v9-4-8",
  "body-large": "SearchInput-module__Djcckw6S__v9-4-8",
  "body-strong-small": "SearchInput-module__gU-eh6ci__v9-4-8",
  "body-strong-base": "SearchInput-module__vqtd6Ftm__v9-4-8",
  "body-strong-large": "SearchInput-module__W2mDSyBR__v9-4-8",
  "page-heading": "SearchInput-module__ZB7RU7Ej__v9-4-8",
  "button": "SearchInput-module__bkymjPcy__v9-4-8",
  "form-label": "SearchInput-module__2G395w7r__v9-4-8",
  "caption": "SearchInput-module__VXKq39-r__v9-4-8",
  "text-link-default": "SearchInput-module__8b4IlhUn__v9-4-8",
  "text-link-hover": "SearchInput-module__MeEXdL8s__v9-4-8",
  "table-data-regular": "SearchInput-module__E7uTC6M0__v9-4-8",
  "table-data-bold": "SearchInput-module__JvKpNgSV__v9-4-8",
  "wrapperBorder": "SearchInput-module__NBttipVL__v9-4-8",
  "icon": "SearchInput-module__NYsD9gcw__v9-4-8"
};
var SearchInput = function (_a) {
  var _b = _a.placeholder,
    placeholder = _b === void 0 ? '' : _b,
    className = _a.className,
    onEscape = _a.onEscape,
    value = _a.value,
    onTextChange = _a.onTextChange,
    _c = _a.searchAutofocus,
    searchAutofocus = _c === void 0 ? true : _c;
  var id = useId('component-');
  var handleChange = useCallback(function (event) {
    return onTextChange(event.currentTarget.value);
  }, [onTextChange]);
  var handleKeyDown = useCallback(function (e) {
    if (e.key === 'Escape') {
      e.preventDefault();
      if (onEscape) onEscape();
    }
  }, [onEscape]);
  return /*#__PURE__*/React.createElement(TextInput, {
    autoFocus: searchAutofocus,
    className: classnames(styles$8.wrapperBorder, className),
    role: "search",
    defaultValue: value,
    id: id,
    onChange: handleChange,
    onKeyDown: handleKeyDown,
    placeholder: placeholder,
    type: "search",
    prefix: /*#__PURE__*/React.createElement(Icon, {
      className: styles$8.icon,
      color: "icon-subdued",
      name: "search"
    })
  });
};
SearchInput.displayName = 'SearchInput';
var styles$7 = {
  "submitFooterContainer": "SubmitFooter-module__VjB4ndjg__v9-4-8",
  "cancelButton": "SubmitFooter-module__8ksTdQK-__v9-4-8"
};
var SubmitFooter = function (_a) {
  var onSubmit = _a.onSubmit,
    onCancel = _a.onCancel,
    _b = _a.submitLabel,
    submitLabel = _b === void 0 ? 'Apply' : _b,
    _c = _a.cancelLabel,
    cancelLabel = _c === void 0 ? 'Cancel' : _c,
    passedProps = __rest(_a, ["onSubmit", "onCancel", "submitLabel", "cancelLabel"]);
  var _d = passedProps,
    submitRef = _d.submitRef,
    _e = _d.handleVisibility,
    handleVisibility = _e === void 0 ? noop : _e,
    selectedOptions = _d.selectedOptions;
  useImperativeHandle(submitRef, function () {
    return {
      cancel: function () {
        onCancel();
      }
    };
  }, [onCancel]);
  return /*#__PURE__*/React.createElement("footer", {
    className: styles$7.submitFooterContainer
  }, /*#__PURE__*/React.createElement(Button, {
    variant: "tertiary",
    onClick: function () {
      onCancel();
      handleVisibility();
    },
    className: styles$7.cancelButton
  }, cancelLabel), /*#__PURE__*/React.createElement(Button, {
    variant: "primary",
    onClick: function () {
      onSubmit(selectedOptions);
      handleVisibility();
    }
  }, submitLabel));
};
var styles$6 = {
  "creatableFooterContainer": "AddOptionsFooter-module__s-wwUOYV__v9-4-8",
  "inputWrapper": "AddOptionsFooter-module__E38wyxL2__v9-4-8",
  "colorPicker": "AddOptionsFooter-module__Hh5OwhSr__v9-4-8"
};
var deriveFooterButtonState = function (inputValue, loading) {
  if (inputValue.length === 0) return 'disabled';
  if (loading) return 'loading';
  return 'enabled';
};
function AddOptionsFooter(_a) {
  var createButtonProps = _a.createButtonProps,
    inputProps = _a.inputProps,
    onCreate = _a.onCreate,
    colorPickerOptions = _a.colorPickerOptions,
    _b = _a.multiple,
    multiple = _b === void 0 ? false : _b,
    passedProps = __rest(_a, ["createButtonProps", "inputProps", "onCreate", "colorPickerOptions", "multiple"]);
  var _c = passedProps,
    _d = _c.handleVisibility,
    handleVisibility = _d === void 0 ? noop : _d,
    closeOnSelect = _c.closeOnSelect;
  var _e = useState(''),
    inputValue = _e[0],
    setInputValue = _e[1];
  var _f = useState(colorPickerOptions === null || colorPickerOptions === void 0 ? void 0 : colorPickerOptions.initialColor),
    color = _f[0],
    setColor = _f[1];
  var onAddHandler = function () {
    onCreate(__assign({
      value: inputValue || ''
    }, color ? {
      color: color
    } : {}));
    setInputValue('');
    if (closeOnSelect && !multiple) handleVisibility();
  };
  var onInputChange = function (event) {
    setInputValue(event.target.value);
  };
  return /*#__PURE__*/React.createElement("footer", {
    className: styles$6.creatableFooterContainer
  }, /*#__PURE__*/React.createElement(TextInput, {
    placeholder: inputProps === null || inputProps === void 0 ? void 0 : inputProps.placeholder,
    className: styles$6.inputWrapper,
    onKeyPress: function (e) {
      if (e.key.toLowerCase() === 'enter' && inputValue) onAddHandler();
    },
    metadata: inputProps === null || inputProps === void 0 ? void 0 : inputProps.metadata,
    onChange: onInputChange,
    value: inputValue
  }), colorPickerOptions && /*#__PURE__*/React.createElement("div", {
    className: styles$6.colorPicker
  }, /*#__PURE__*/React.createElement(ColorPicker, {
    closeOnSelect: true,
    initialColor: color,
    metadata: colorPickerOptions.metadata,
    onChange: setColor,
    palette: colorPickerOptions.palette,
    title: colorPickerOptions.title
  })), /*#__PURE__*/React.createElement(Button, {
    variant: "primary",
    onClick: onAddHandler,
    buttonState: deriveFooterButtonState(inputValue, createButtonProps.loading)
  }, createButtonProps.label));
}
var styles$5 = {
  "noResults": "EmptyState-module__Gq49BSwD__v9-4-8",
  "defaultEmptyState": "EmptyState-module__e7sO1npT__v9-4-8"
};
var DefaultEmptyState = function (_a) {
  var label = _a.label,
    searchNoResultsLabel = _a.searchNoResultsLabel;
  return /*#__PURE__*/React.createElement(Typography, {
    component: "div",
    token: "body-base",
    className: searchNoResultsLabel ? styles$5.noResults : styles$5.defaultEmptyState
  }, searchNoResultsLabel ? searchNoResultsLabel + " \"" + label + "\"" : label);
};
var styles$4 = {
  "heading-xsmall": "DefaultSearchInput-module__XqAvbSH4__v9-4-8",
  "heading-small": "DefaultSearchInput-module__PjZaoB1D__v9-4-8",
  "heading-medium": "DefaultSearchInput-module__9YfBCF4j__v9-4-8",
  "heading-large": "DefaultSearchInput-module__cIWfgi6z__v9-4-8",
  "heading-xlarge": "DefaultSearchInput-module__UeIs6I5u__v9-4-8",
  "heading-2xl": "DefaultSearchInput-module__BfyeFbkq__v9-4-8",
  "body-small": "DefaultSearchInput-module__caCyv9Nc__v9-4-8",
  "body-base": "DefaultSearchInput-module__K4tdeiTw__v9-4-8",
  "wrapper": "DefaultSearchInput-module__3-zVyXUS__v9-4-8",
  "body-large": "DefaultSearchInput-module__UO-1limM__v9-4-8",
  "body-strong-small": "DefaultSearchInput-module__H6XXoy63__v9-4-8",
  "body-strong-base": "DefaultSearchInput-module__-tNqTN0-__v9-4-8",
  "body-strong-large": "DefaultSearchInput-module__CGPNjXP3__v9-4-8",
  "page-heading": "DefaultSearchInput-module__Bdvh1-pz__v9-4-8",
  "button": "DefaultSearchInput-module__1NkKn7GK__v9-4-8",
  "form-label": "DefaultSearchInput-module__CQr816k9__v9-4-8",
  "caption": "DefaultSearchInput-module__bvrmydjy__v9-4-8",
  "text-link-default": "DefaultSearchInput-module__bpRe-gpx__v9-4-8",
  "text-link-hover": "DefaultSearchInput-module__uFVODikI__v9-4-8",
  "table-data-regular": "DefaultSearchInput-module__5BIV1YqJ__v9-4-8",
  "table-data-bold": "DefaultSearchInput-module__Xy4SBgBk__v9-4-8",
  "icon": "DefaultSearchInput-module__WdsciYG8__v9-4-8"
};
var SearchInputWithRef = function (_a, ref) {
  var _b = _a.placeholder,
    placeholder = _b === void 0 ? '' : _b,
    value = _a.value,
    onTextChange = _a.onTextChange,
    onEscape = _a.onEscape,
    _c = _a.searchAutofocus,
    searchAutofocus = _c === void 0 ? true : _c;
  var id = useId('component-');
  var inputRef = useRef(null);
  var finalInputRef = useForkRef([ref, inputRef]);
  useEffect(function () {
    if (inputRef.current && searchAutofocus) {
      inputRef.current.focus();
    }
  }, [ref, searchAutofocus]);
  var handleChange = function (event) {
    return onTextChange(event.currentTarget.value);
  };
  var handleKeyDown = function (e) {
    if (e.key === 'Escape') {
      e.preventDefault();
      if (onEscape) onEscape();
    }
  };
  return /*#__PURE__*/React.createElement("div", {
    className: styles$4.wrapper,
    role: "search"
  }, /*#__PURE__*/React.createElement("label", {
    htmlFor: id
  }, /*#__PURE__*/React.createElement(Icon, {
    className: styles$4.icon,
    color: "icon-subdued",
    name: "search",
    size: 12
  }), /*#__PURE__*/React.createElement("input", {
    defaultValue: value,
    id: id,
    onChange: handleChange,
    onKeyDown: handleKeyDown,
    placeholder: placeholder,
    ref: finalInputRef,
    type: "search"
  })));
};
SearchInputWithRef.displayName = 'SearchInput';
var DefaultSearchInput = /*#__PURE__*/React.forwardRef(SearchInputWithRef);
var SelectSearchInput = function (_a) {
  var enableNewSearchInput = _a.enableNewSearchInput,
    rest = __rest(_a, ["enableNewSearchInput"]);
  var _b = useContext(SelectContext),
    shouldShowSearchInput = _b.shouldShowSearchInput,
    searchInputValue = _b.searchInputValue,
    setSearchInputValue = _b.setSearchInputValue;
  var SelectedSearchInput = enableNewSearchInput ? SearchInput : DefaultSearchInput;
  return shouldShowSearchInput ? /*#__PURE__*/React.createElement(SelectedSearchInput, __assign({}, rest, {
    value: searchInputValue,
    onTextChange: setSearchInputValue
  })) : null;
};
SelectSearchInput.displayName = 'SearchInput';
var isScrollableElement = function (el) {
  return el.clientHeight !== el.scrollHeight;
};
var SelectContent = function (_a) {
  var _b;
  var className = _a.className,
    _c = _a.closeOnSelect,
    closeOnSelect = _c === void 0 ? false : _c,
    children = _a.children,
    selectTriggerRef = _a.selectTriggerRef,
    props = __rest(_a, ["className", "closeOnSelect", "children", "selectTriggerRef"]);
  var submitRef = useRef(null);
  var listBoxRef = useRef(null);
  var _d = useContext(SelectContext),
    isLoading = _d.isLoading,
    multiple = _d.multiple,
    searchInputValue = _d.searchInputValue,
    shouldShowNoResultsLabel = _d.shouldShowNoResultsLabel,
    selectedOptionsWithValuesOnly = _d.selectedOptionsWithValuesOnly,
    selectedOptions = _d.selectedOptions,
    options = _d.options,
    setIsVisible = _d.setIsVisible,
    isFullWidthVariant = _d.isFullWidthVariant,
    optionType = _d.optionType,
    onSelect = _d.onSelect;
  useLayoutEffect(function () {
    var el = listBoxRef.current;
    if (!el) {
      return;
    }
    if (isScrollableElement(el)) {
      el.classList.add(styles$9.listboxOverflowScroll);
    } else {
      el.classList.remove(styles$9.listboxOverflowScroll);
    }
  }, [listBoxRef]);
  var handleEscape = useCallback(function () {
    setIsVisible(false);
    // hacky way to focus trigger once user uses "Escape" key on the content part
    setTimeout(function () {
      if (selectTriggerRef === null || selectTriggerRef === void 0 ? void 0 : selectTriggerRef.current) {
        var triggerContainer = selectTriggerRef.current.querySelector('[tabIndex="0"]');
        if (triggerContainer) triggerContainer === null || triggerContainer === void 0 ? void 0 : triggerContainer.focus();
      }
    }, 0);
    if (submitRef.current) {
      submitRef.current.cancel();
    }
  }, [selectTriggerRef, setIsVisible]);
  var handleSelectAll = useCallback(function (selected) {
    var opts = selected ? getAllSelectableOptions(options, selectedOptions) : getFixedOptions(options, selectedOptions);
    onSelect(opts);
  }, [onSelect, options, selectedOptions]);
  var handleSelect = useCallback(function (option) {
    onSelect(getSelectedValues({
      option: option,
      selectedOptions: selectedOptions,
      selectedOptionsWithValuesOnly: selectedOptionsWithValuesOnly || [],
      multiple: multiple
    }));
  }, [multiple, onSelect, selectedOptions, selectedOptionsWithValuesOnly]);
  var noOptions = options.length === 0;
  return /*#__PURE__*/React.createElement(Popover.Content, __assign({}, props, {
    className: classnames(styles$9.list, (_b = {}, _b[styles$9.multiLineList] = optionType === 'multi-line', _b[styles$9.fullWidth] = multiple || isFullWidthVariant, _b), className)
  }), isLoading ? /*#__PURE__*/React.createElement("div", {
    className: styles$9.loadingContainer,
    "aria-label": "loading"
  }, /*#__PURE__*/React.createElement(Icon, {
    name: "spinner-third",
    size: 16,
    spin: true,
    color: "icon-highlight",
    type: "solid"
  })) : React.Children.map(children, function (listItem) {
    if (! /*#__PURE__*/React.isValidElement(listItem)) {
      return null;
    }
    switch (listItem.type.displayName) {
      case SearchInput.displayName:
        return /*#__PURE__*/React.cloneElement(listItem, {
          onEscape: handleEscape
        });
      case ListBox.displayName:
        return /*#__PURE__*/React.createElement(React.Fragment, null, !shouldShowNoResultsLabel && !noOptions && /*#__PURE__*/React.cloneElement(listItem, {
          listRef: listBoxRef,
          onEscape: handleEscape,
          handleSelect: function (data) {
            handleSelect(data);
            if (closeOnSelect) setIsVisible(false);
          },
          handleSelectAll: function (data) {
            handleSelectAll(data);
            if (closeOnSelect) setIsVisible(false);
          },
          optionType: optionType
        }));
    }
    switch (listItem.type) {
      case AddOptionsFooter:
        return /*#__PURE__*/React.cloneElement(listItem, {
          handleVisibility: function () {
            return setIsVisible(false);
          },
          closeOnSelect: closeOnSelect
        });
      case SubmitFooter:
        return /*#__PURE__*/React.cloneElement(listItem, {
          handleVisibility: function () {
            return setIsVisible(false);
          },
          submitRef: submitRef,
          selectedOptions: selectedOptions
        });
      case DefaultEmptyState:
        if (listItem.props.customComponent) {
          return listItem.props.customComponent();
        }
        return /*#__PURE__*/React.createElement(React.Fragment, null, shouldShowNoResultsLabel ? /*#__PURE__*/React.cloneElement(listItem, {
          label: searchInputValue
        }) : noOptions && /*#__PURE__*/React.cloneElement(listItem, {
          searchNoResultsLabel: undefined
        }));
      default:
        return null;
    }
  }));
};
SelectContent.displayName = 'Content';
SelectContent.SearchInput = SelectSearchInput;
SelectContent.DefaultEmptyState = DefaultEmptyState;
SelectContent.AddOptionsFooter = AddOptionsFooter;
SelectContent.SubmitFooter = SubmitFooter;
SelectContent.ListBox = SelectListBox;
var SelectTrigger = function (_a) {
  var _b, _c, _d;
  var children = _a.children,
    size = _a.size,
    disabled = _a.disabled,
    props = __rest(_a, ["children", "size", "disabled"]);
  var _e = useContext(SelectContext),
    multiple = _e.multiple,
    selectedOptions = _e.selectedOptions,
    setIsVisible = _e.setIsVisible,
    onSelect = _e.onSelect,
    options = _e.options,
    isClearable = _e.isClearable,
    isFullWidthVariant = _e.isFullWidthVariant,
    selectedOptionsWithValuesOnly = _e.selectedOptionsWithValuesOnly;
  var handleClear = function (evt) {
    evt.stopPropagation();
    var fixedOptions = getFixedOptions(options, selectedOptions);
    onSelect(fixedOptions);
    setIsVisible(false);
  };
  var handleRemove = function (option) {
    onSelect(getSelectedValues({
      option: option,
      selectedOptions: selectedOptions,
      selectedOptionsWithValuesOnly: selectedOptionsWithValuesOnly || [],
      multiple: multiple
    }));
  };
  useEffect(function () {
    if (disabled) {
      setIsVisible(false);
    }
  }, [disabled, setIsVisible]);
  return /*#__PURE__*/React.createElement(Popover.Trigger, __assign({
    multiple: multiple
  }, props, {
    disabled: disabled,
    selectedOptions: selectedOptions,
    setListVisibility: setIsVisible,
    onClear: isClearable ? handleClear : undefined,
    onRemove: multiple ? handleRemove : undefined,
    size: isFullWidthVariant ? undefined : size !== null && size !== void 0 ? size : ((_c = (_b = selectedOptions[0]) === null || _b === void 0 ? void 0 : _b.label) === null || _c === void 0 ? void 0 : _c.length) || ((_d = props.label) === null || _d === void 0 ? void 0 : _d.length)
  }), children);
};
SelectTrigger.displayName = 'Trigger';
var BaseSelect = /*#__PURE__*/React.forwardRef(function (_a, ref) {
  var selectedOptions = _a.selectedOptions,
    minOptionsToDisplaySearch = _a.minOptionsToDisplaySearch,
    variant = _a.variant,
    optionType = _a.optionType,
    _b = _a.onSelect,
    onSelect = _b === void 0 ? noop : _b,
    multiple = _a.multiple,
    isClearable = _a.isClearable,
    isLoading = _a.isLoading,
    searchInitialValue = _a.searchInitialValue,
    onSearchChange = _a.onSearchChange,
    children = _a.children,
    restProps = __rest(_a, ["selectedOptions", "minOptionsToDisplaySearch", "variant", "optionType", "onSelect", "multiple", "isClearable", "isLoading", "searchInitialValue", "onSearchChange", "children"]);
  return /*#__PURE__*/React.createElement(SelectProvider, {
    selectedOptions: selectedOptions,
    minOptionsToDisplaySearch: minOptionsToDisplaySearch,
    variant: variant,
    optionType: optionType,
    onSelect: onSelect,
    multiple: multiple,
    isClearable: isClearable,
    isLoading: isLoading,
    searchInitialValue: searchInitialValue,
    onSearchChange: onSearchChange
  }, /*#__PURE__*/React.createElement(SelectPopover, __assign({}, restProps, {
    variant: variant,
    ref: Object.keys(ref || {}).length > 0 ? ref : null
  }), children));
});
BaseSelect.Trigger = SelectTrigger;
BaseSelect.Content = SelectContent;
var generateOptionsAndOptionGroups = function (options) {
  var groupedOptions = groupBy(options, function (option) {
    return option.group;
  });
  var shouldShowGroups = Object.keys(groupedOptions).length > 1;
  var renderOptions = function (optionsList) {
    return optionsList.map(function (option) {
      return /*#__PURE__*/React.createElement(BaseSelect.Content.ListBox.Option, __assign({
        key: option.value
      }, option, {
        value: option.value,
        disabled: option.disabled
      }), option.label);
    });
  };
  return shouldShowGroups ? lodash_topairs(groupedOptions).map(function (_a) {
    var groupName = _a[0],
      groupOptions = _a[1];
    return /*#__PURE__*/React.createElement(BaseSelect.Content.ListBox.OptionGroup, {
      key: groupName,
      name: groupName
    }, renderOptions(groupOptions));
  }) : renderOptions(options);
};
var styles$3 = {
  "selectTrigger": "DefaultTrigger-module__EFr4EJbn__v9-4-8",
  "fullWidth": "DefaultTrigger-module__ywwNUB8z__v9-4-8",
  "innerText": "DefaultTrigger-module__UUopHdDJ__v9-4-8",
  "outline-warning": "DefaultTrigger-module__ijTAHA63__v9-4-8",
  "outline-error": "DefaultTrigger-module__J2JSuhwR__v9-4-8",
  "disabled": "DefaultTrigger-module__tkVBeoVj__v9-4-8",
  "selectTriggerIcons": "DefaultTrigger-module__sY97qx90__v9-4-8",
  "selectClearButton": "DefaultTrigger-module__Nlp-NE-h__v9-4-8",
  "icon": "DefaultTrigger-module__c7zJm5M2__v9-4-8"
};
var DefaultTrigger = /*#__PURE__*/React.forwardRef(function (_a, ref) {
  var _b;
  var disabled = _a.disabled,
    label = _a.label,
    _c = _a.outline,
    outline = _c === void 0 ? 'default' : _c,
    _d = _a.autoFocus,
    autoFocus = _d === void 0 ? false : _d,
    triggerId = _a.triggerId,
    variant = _a.variant,
    visible = _a.visible,
    _e = _a.onClick,
    onClick = _e === void 0 ? noop : _e,
    _f = _a.selectedOptions,
    selectedOptions = _f === void 0 ? [] : _f,
    _g = _a.multiple,
    multiple = _g === void 0 ? false : _g,
    onClear = _a.onClear;
  var hasSelectedOption = selectedOptions.length > 0;
  var isFullWidth = variant === 'full-width';
  var firstItem = selectedOptions[0];
  var singleSelectionLabel = hasSelectedOption ? firstItem.label : label;
  var multiSelectionLabel = hasSelectedOption ? label + " \u00B7 " + selectedOptions.length : label;
  var handleKeyDown = function (e) {
    if ([' ', 'Enter'].some(function (k) {
      return k === e.key;
    })) {
      e.preventDefault();
      onClick();
    }
  };
  return /*#__PURE__*/React.createElement(Typography, {
    ref: ref,
    component: "button",
    token: "body-base",
    className: classnames(styles$3.selectTrigger, styles$3["outline-" + outline], (_b = {}, _b[styles$3.fullWidth] = isFullWidth, _b[styles$3.disabled] = disabled, _b)),
    role: "button",
    disabled: disabled,
    onClick: onClick,
    onKeyDown: handleKeyDown,
    autoFocus: autoFocus,
    id: triggerId,
    tabIndex: 0
  }, /*#__PURE__*/React.createElement("span", {
    className: styles$3.innerText
  }, multiple ? multiSelectionLabel : singleSelectionLabel), /*#__PURE__*/React.createElement("div", {
    className: styles$3.selectTriggerIcons
  }, onClear && selectedOptions.length > 0 ? /*#__PURE__*/React.createElement("div", {
    role: "button",
    className: styles$3.selectClearButton,
    onClick: disabled ? noop : onClear,
    onKeyPress: disabled ? noop : onClear,
    tabIndex: -1,
    "aria-label": "Clear Select Input"
  }, /*#__PURE__*/React.createElement(Icon, {
    name: "times-circle",
    type: "solid",
    color: "icon-subdued"
  })) : null, /*#__PURE__*/React.createElement("div", {
    className: styles$3.icon
  }, /*#__PURE__*/React.createElement(Icon, {
    name: visible ? 'angle-up' : 'angle-down',
    type: "solid",
    color: "icon-default"
  }))));
});
var styles$2 = {
  "heading-xsmall": "InputTrigger-module__W1pngG2U__v9-4-8",
  "heading-small": "InputTrigger-module__Pkq6w9dK__v9-4-8",
  "heading-medium": "InputTrigger-module__LEK8eESh__v9-4-8",
  "heading-large": "InputTrigger-module__r0lOq9Tn__v9-4-8",
  "heading-xlarge": "InputTrigger-module__gMbASns9__v9-4-8",
  "heading-2xl": "InputTrigger-module__nl4CXzeN__v9-4-8",
  "body-small": "InputTrigger-module__5xft2WXg__v9-4-8",
  "body-base": "InputTrigger-module__q3WztU-q__v9-4-8",
  "selectTrigger": "InputTrigger-module__300wddmh__v9-4-8",
  "input": "InputTrigger-module__HCaE0xFK__v9-4-8",
  "body-large": "InputTrigger-module__wOYesm1E__v9-4-8",
  "body-strong-small": "InputTrigger-module__-HoOP6at__v9-4-8",
  "body-strong-base": "InputTrigger-module__EbY6p6qb__v9-4-8",
  "body-strong-large": "InputTrigger-module__Xb17YREH__v9-4-8",
  "page-heading": "InputTrigger-module__hBVvaSnZ__v9-4-8",
  "button": "InputTrigger-module__euBryAYj__v9-4-8",
  "form-label": "InputTrigger-module__Fl-A7MzB__v9-4-8",
  "caption": "InputTrigger-module__ppDrLnvj__v9-4-8",
  "text-link-default": "InputTrigger-module__EpE3P2wX__v9-4-8",
  "text-link-hover": "InputTrigger-module__3q07g8Dj__v9-4-8",
  "table-data-regular": "InputTrigger-module__f0837VAT__v9-4-8",
  "table-data-bold": "InputTrigger-module__SYMR2NzS__v9-4-8",
  "selected": "InputTrigger-module__mx690JiD__v9-4-8",
  "fullWidth": "InputTrigger-module__6O-TPsfM__v9-4-8",
  "disabled": "InputTrigger-module__b47iAFO7__v9-4-8",
  "icon": "InputTrigger-module__D0M4AvyN__v9-4-8"
};
var InputTrigger = /*#__PURE__*/React.forwardRef(function (_a, ref) {
  var _b, _c;
  var disabled = _a.disabled,
    label = _a.label,
    _d = _a.outline,
    outline = _d === void 0 ? 'default' : _d,
    _e = _a.autoFocus,
    autoFocus = _e === void 0 ? false : _e,
    triggerId = _a.triggerId,
    variant = _a.variant,
    visible = _a.visible,
    _f = _a.onClick,
    onClick = _f === void 0 ? noop : _f,
    _g = _a.selectedOptions,
    selectedOptions = _g === void 0 ? [] : _g,
    size = _a.size;
  var hasSelectedOption = selectedOptions.length > 0;
  var firstItem = selectedOptions[0];
  var handleKeyDown = function (e) {
    if ([' ', 'Enter'].some(function (k) {
      return k === e.key;
    })) {
      e.preventDefault();
      onClick();
    }
  };
  return /*#__PURE__*/React.createElement(TextInput, {
    containerRef: ref,
    className: classnames(styles$2.selectTrigger, (_b = {}, _b[styles$2.fullWidth] = variant === 'full-width', _b[styles$2.disabled] = disabled, _b)),
    inputClassName: classnames(styles$2.input, (_c = {}, _c[styles$2.selected] = hasSelectedOption, _c)),
    id: triggerId,
    placeholder: label,
    onClick: onClick,
    size: size || 0,
    onKeyDown: handleKeyDown,
    autoFocus: autoFocus,
    outline: outline,
    value: hasSelectedOption ? firstItem.label : label,
    disabled: disabled,
    suffix: /*#__PURE__*/React.createElement(IconButton, {
      icon: visible ? 'angle-up' : 'angle-down',
      iconColor: !disabled ? 'icon-default' : 'icon-disabled',
      variant: "tertiary",
      tabIndex: -1,
      "aria-label": "Toggle visibility",
      className: styles$2.icon,
      buttonState: disabled ? 'disabled' : 'enabled'
    }),
    readOnly: true,
    tabIndex: 0
  });
});
var styles$1 = {
  "heading-xsmall": "MultiInputTrigger-module__yeL2h9oW__v9-4-8",
  "heading-small": "MultiInputTrigger-module__PDr18c-z__v9-4-8",
  "heading-medium": "MultiInputTrigger-module__w0t4bJoz__v9-4-8",
  "heading-large": "MultiInputTrigger-module__NNjjHgHJ__v9-4-8",
  "heading-xlarge": "MultiInputTrigger-module__vwxJVTMh__v9-4-8",
  "heading-2xl": "MultiInputTrigger-module__sx8pwzqS__v9-4-8",
  "body-small": "MultiInputTrigger-module__htuysFZP__v9-4-8",
  "body-base": "MultiInputTrigger-module__ezqdcPBt__v9-4-8",
  "selectTrigger": "MultiInputTrigger-module__-KI-nYAG__v9-4-8",
  "chipsContainer": "MultiInputTrigger-module__kbcdO30j__v9-4-8",
  "inputElement": "MultiInputTrigger-module__EJlErVjx__v9-4-8",
  "body-large": "MultiInputTrigger-module__6XPlcUgn__v9-4-8",
  "body-strong-small": "MultiInputTrigger-module__pzUO8cOf__v9-4-8",
  "body-strong-base": "MultiInputTrigger-module__HqrmuEZT__v9-4-8",
  "body-strong-large": "MultiInputTrigger-module__JtaLDJh8__v9-4-8",
  "page-heading": "MultiInputTrigger-module__XojbYDIV__v9-4-8",
  "button": "MultiInputTrigger-module__AkBhtyVv__v9-4-8",
  "form-label": "MultiInputTrigger-module__vm3zSrnz__v9-4-8",
  "caption": "MultiInputTrigger-module__z1R-Sgny__v9-4-8",
  "text-link-default": "MultiInputTrigger-module__onChF5-C__v9-4-8",
  "text-link-hover": "MultiInputTrigger-module__jKSEOuEg__v9-4-8",
  "table-data-regular": "MultiInputTrigger-module__k1UBcn89__v9-4-8",
  "table-data-bold": "MultiInputTrigger-module__k2WiFv5w__v9-4-8",
  "limitHeight": "MultiInputTrigger-module__moxdqa4l__v9-4-8",
  "focused": "MultiInputTrigger-module__9jgKH7GZ__v9-4-8",
  "hideContent": "MultiInputTrigger-module__9zLJRYf9__v9-4-8",
  "isHidden": "MultiInputTrigger-module__MDoDhXut__v9-4-8",
  "fullWidth": "MultiInputTrigger-module__Xo-jRzU5__v9-4-8",
  "disabled": "MultiInputTrigger-module__HoJWZEfD__v9-4-8",
  "selectTriggerIcons": "MultiInputTrigger-module__xnQ4jQri__v9-4-8",
  "icon": "MultiInputTrigger-module__XdfmFhky__v9-4-8"
};
var styles = {
  "selectChip": "SelectChip-module__Qn8YpwPn__v9-4-8",
  "removableSelectChip": "SelectChip-module__a4rKKNia__v9-4-8",
  "clickableSelectChip": "SelectChip-module__N79OIUl9__v9-4-8",
  "removeIcon": "SelectChip-module__f1FhGayq__v9-4-8"
};
var SelectChip = /*#__PURE__*/React.forwardRef(function (_a, ref) {
  var _b;
  var children = _a.children,
    _c = _a.disabled,
    disabled = _c === void 0 ? false : _c,
    onRemove = _a.onRemove,
    onKeyDown = _a.onKeyDown,
    onClick = _a.onClick,
    className = _a.className,
    metadata = _a.metadata,
    _d = _a.tabIndex,
    tabIndex = _d === void 0 ? 0 : _d;
  var isRemovable = !disabled && onRemove !== undefined;
  var isClickable = !disabled && onClick !== undefined;
  var chipAttributes = isRemovable || isClickable ? {
    role: 'button',
    tabIndex: tabIndex,
    onKeyDown: function (event) {
      switch (event.key) {
        case 'Delete':
        case 'Backspace':
          if (onRemove) {
            onRemove(event);
          }
          break;
        default:
          if (onKeyDown) {
            onKeyDown(event);
          }
          break;
      }
    },
    onClick: onClick
  } : {};
  return /*#__PURE__*/React.createElement("div", __assign({}, parseMetadata(metadata), chipAttributes, {
    ref: ref,
    className: classnames(styles.selectChip, (_b = {}, _b[styles.clickableSelectChip] = isClickable, _b[styles.removableSelectChip] = isRemovable, _b), className),
    "aria-disabled": disabled
  }), /*#__PURE__*/React.createElement(Typography, {
    component: "span",
    token: "body-base",
    color: disabled ? 'text-disabled' : 'text-default'
  }, children), isRemovable && /*#__PURE__*/React.createElement("span", {
    className: styles.removeIcon,
    onClick: onRemove,
    "aria-hidden": true
  }, /*#__PURE__*/React.createElement(Icon, {
    metadata: suffixMetadata(metadata, 'remove-icon'),
    name: "times",
    size: 10
  })));
});

/**
 * lodash (Custom Build) <https://lodash.com/>
 * Build: `lodash modularize exports="npm" -o ./`
 * Copyright jQuery Foundation and other contributors <https://jquery.org/>
 * Released under MIT license <https://lodash.com/license>
 * Based on Underscore.js 1.8.3 <http://underscorejs.org/LICENSE>
 * Copyright Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
 */

/** Used as the `TypeError` message for "Functions" methods. */
var FUNC_ERROR_TEXT = 'Expected a function';

/** Used as references for various `Number` constants. */
var NAN = 0 / 0;

/** `Object#toString` result references. */
var symbolTag = '[object Symbol]';

/** Used to match leading and trailing whitespace. */
var reTrim = /^\s+|\s+$/g;

/** Used to detect bad signed hexadecimal string values. */
var reIsBadHex = /^[-+]0x[0-9a-f]+$/i;

/** Used to detect binary string values. */
var reIsBinary = /^0b[01]+$/i;

/** Used to detect octal string values. */
var reIsOctal = /^0o[0-7]+$/i;

/** Built-in method references without a dependency on `root`. */
var freeParseInt = parseInt;

/** Detect free variable `global` from Node.js. */
var freeGlobal = typeof commonjsGlobal == 'object' && commonjsGlobal && commonjsGlobal.Object === Object && commonjsGlobal;

/** Detect free variable `self`. */
var freeSelf = typeof self == 'object' && self && self.Object === Object && self;

/** Used as a reference to the global object. */
var root = freeGlobal || freeSelf || Function('return this')();

/** Used for built-in method references. */
var objectProto = Object.prototype;

/**
 * Used to resolve the
 * [`toStringTag`](http://ecma-international.org/ecma-262/7.0/#sec-object.prototype.tostring)
 * of values.
 */
var objectToString = objectProto.toString;

/* Built-in method references for those with the same name as other `lodash` methods. */
var nativeMax = Math.max,
  nativeMin = Math.min;

/**
 * Gets the timestamp of the number of milliseconds that have elapsed since
 * the Unix epoch (1 January 1970 00:00:00 UTC).
 *
 * @static
 * @memberOf _
 * @since 2.4.0
 * @category Date
 * @returns {number} Returns the timestamp.
 * @example
 *
 * _.defer(function(stamp) {
 *   console.log(_.now() - stamp);
 * }, _.now());
 * // => Logs the number of milliseconds it took for the deferred invocation.
 */
var now = function () {
  return root.Date.now();
};

/**
 * Creates a debounced function that delays invoking `func` until after `wait`
 * milliseconds have elapsed since the last time the debounced function was
 * invoked. The debounced function comes with a `cancel` method to cancel
 * delayed `func` invocations and a `flush` method to immediately invoke them.
 * Provide `options` to indicate whether `func` should be invoked on the
 * leading and/or trailing edge of the `wait` timeout. The `func` is invoked
 * with the last arguments provided to the debounced function. Subsequent
 * calls to the debounced function return the result of the last `func`
 * invocation.
 *
 * **Note:** If `leading` and `trailing` options are `true`, `func` is
 * invoked on the trailing edge of the timeout only if the debounced function
 * is invoked more than once during the `wait` timeout.
 *
 * If `wait` is `0` and `leading` is `false`, `func` invocation is deferred
 * until to the next tick, similar to `setTimeout` with a timeout of `0`.
 *
 * See [David Corbacho's article](https://css-tricks.com/debouncing-throttling-explained-examples/)
 * for details over the differences between `_.debounce` and `_.throttle`.
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Function
 * @param {Function} func The function to debounce.
 * @param {number} [wait=0] The number of milliseconds to delay.
 * @param {Object} [options={}] The options object.
 * @param {boolean} [options.leading=false]
 *  Specify invoking on the leading edge of the timeout.
 * @param {number} [options.maxWait]
 *  The maximum time `func` is allowed to be delayed before it's invoked.
 * @param {boolean} [options.trailing=true]
 *  Specify invoking on the trailing edge of the timeout.
 * @returns {Function} Returns the new debounced function.
 * @example
 *
 * // Avoid costly calculations while the window size is in flux.
 * jQuery(window).on('resize', _.debounce(calculateLayout, 150));
 *
 * // Invoke `sendMail` when clicked, debouncing subsequent calls.
 * jQuery(element).on('click', _.debounce(sendMail, 300, {
 *   'leading': true,
 *   'trailing': false
 * }));
 *
 * // Ensure `batchLog` is invoked once after 1 second of debounced calls.
 * var debounced = _.debounce(batchLog, 250, { 'maxWait': 1000 });
 * var source = new EventSource('/stream');
 * jQuery(source).on('message', debounced);
 *
 * // Cancel the trailing debounced invocation.
 * jQuery(window).on('popstate', debounced.cancel);
 */
function debounce(func, wait, options) {
  var lastArgs,
    lastThis,
    maxWait,
    result,
    timerId,
    lastCallTime,
    lastInvokeTime = 0,
    leading = false,
    maxing = false,
    trailing = true;
  if (typeof func != 'function') {
    throw new TypeError(FUNC_ERROR_TEXT);
  }
  wait = toNumber(wait) || 0;
  if (isObject(options)) {
    leading = !!options.leading;
    maxing = 'maxWait' in options;
    maxWait = maxing ? nativeMax(toNumber(options.maxWait) || 0, wait) : maxWait;
    trailing = 'trailing' in options ? !!options.trailing : trailing;
  }
  function invokeFunc(time) {
    var args = lastArgs,
      thisArg = lastThis;
    lastArgs = lastThis = undefined;
    lastInvokeTime = time;
    result = func.apply(thisArg, args);
    return result;
  }
  function leadingEdge(time) {
    // Reset any `maxWait` timer.
    lastInvokeTime = time;
    // Start the timer for the trailing edge.
    timerId = setTimeout(timerExpired, wait);
    // Invoke the leading edge.
    return leading ? invokeFunc(time) : result;
  }
  function remainingWait(time) {
    var timeSinceLastCall = time - lastCallTime,
      timeSinceLastInvoke = time - lastInvokeTime,
      result = wait - timeSinceLastCall;
    return maxing ? nativeMin(result, maxWait - timeSinceLastInvoke) : result;
  }
  function shouldInvoke(time) {
    var timeSinceLastCall = time - lastCallTime,
      timeSinceLastInvoke = time - lastInvokeTime;

    // Either this is the first call, activity has stopped and we're at the
    // trailing edge, the system time has gone backwards and we're treating
    // it as the trailing edge, or we've hit the `maxWait` limit.
    return lastCallTime === undefined || timeSinceLastCall >= wait || timeSinceLastCall < 0 || maxing && timeSinceLastInvoke >= maxWait;
  }
  function timerExpired() {
    var time = now();
    if (shouldInvoke(time)) {
      return trailingEdge(time);
    }
    // Restart the timer.
    timerId = setTimeout(timerExpired, remainingWait(time));
  }
  function trailingEdge(time) {
    timerId = undefined;

    // Only invoke if we have `lastArgs` which means `func` has been
    // debounced at least once.
    if (trailing && lastArgs) {
      return invokeFunc(time);
    }
    lastArgs = lastThis = undefined;
    return result;
  }
  function cancel() {
    if (timerId !== undefined) {
      clearTimeout(timerId);
    }
    lastInvokeTime = 0;
    lastArgs = lastCallTime = lastThis = timerId = undefined;
  }
  function flush() {
    return timerId === undefined ? result : trailingEdge(now());
  }
  function debounced() {
    var time = now(),
      isInvoking = shouldInvoke(time);
    lastArgs = arguments;
    lastThis = this;
    lastCallTime = time;
    if (isInvoking) {
      if (timerId === undefined) {
        return leadingEdge(lastCallTime);
      }
      if (maxing) {
        // Handle invocations in a tight loop.
        timerId = setTimeout(timerExpired, wait);
        return invokeFunc(lastCallTime);
      }
    }
    if (timerId === undefined) {
      timerId = setTimeout(timerExpired, wait);
    }
    return result;
  }
  debounced.cancel = cancel;
  debounced.flush = flush;
  return debounced;
}

/**
 * Checks if `value` is the
 * [language type](http://www.ecma-international.org/ecma-262/7.0/#sec-ecmascript-language-types)
 * of `Object`. (e.g. arrays, functions, objects, regexes, `new Number(0)`, and `new String('')`)
 *
 * @static
 * @memberOf _
 * @since 0.1.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is an object, else `false`.
 * @example
 *
 * _.isObject({});
 * // => true
 *
 * _.isObject([1, 2, 3]);
 * // => true
 *
 * _.isObject(_.noop);
 * // => true
 *
 * _.isObject(null);
 * // => false
 */
function isObject(value) {
  var type = typeof value;
  return !!value && (type == 'object' || type == 'function');
}

/**
 * Checks if `value` is object-like. A value is object-like if it's not `null`
 * and has a `typeof` result of "object".
 *
 * @static
 * @memberOf _
 * @since 4.0.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is object-like, else `false`.
 * @example
 *
 * _.isObjectLike({});
 * // => true
 *
 * _.isObjectLike([1, 2, 3]);
 * // => true
 *
 * _.isObjectLike(_.noop);
 * // => false
 *
 * _.isObjectLike(null);
 * // => false
 */
function isObjectLike(value) {
  return !!value && typeof value == 'object';
}

/**
 * Checks if `value` is classified as a `Symbol` primitive or object.
 *
 * @static
 * @memberOf _
 * @since 4.0.0
 * @category Lang
 * @param {*} value The value to check.
 * @returns {boolean} Returns `true` if `value` is a symbol, else `false`.
 * @example
 *
 * _.isSymbol(Symbol.iterator);
 * // => true
 *
 * _.isSymbol('abc');
 * // => false
 */
function isSymbol(value) {
  return typeof value == 'symbol' || isObjectLike(value) && objectToString.call(value) == symbolTag;
}

/**
 * Converts `value` to a number.
 *
 * @static
 * @memberOf _
 * @since 4.0.0
 * @category Lang
 * @param {*} value The value to process.
 * @returns {number} Returns the number.
 * @example
 *
 * _.toNumber(3.2);
 * // => 3.2
 *
 * _.toNumber(Number.MIN_VALUE);
 * // => 5e-324
 *
 * _.toNumber(Infinity);
 * // => Infinity
 *
 * _.toNumber('3.2');
 * // => 3.2
 */
function toNumber(value) {
  if (typeof value == 'number') {
    return value;
  }
  if (isSymbol(value)) {
    return NAN;
  }
  if (isObject(value)) {
    var other = typeof value.valueOf == 'function' ? value.valueOf() : value;
    value = isObject(other) ? other + '' : other;
  }
  if (typeof value != 'string') {
    return value === 0 ? value : +value;
  }
  value = value.replace(reTrim, '');
  var isBinary = reIsBinary.test(value);
  return isBinary || reIsOctal.test(value) ? freeParseInt(value.slice(2), isBinary ? 2 : 8) : reIsBadHex.test(value) ? NAN : +value;
}
var lodash_debounce = debounce;
var useResizeObserver = function () {
  var _a = useState({
      width: window.innerWidth,
      height: window.innerHeight
    }),
    dimensions = _a[0],
    setDimensions = _a[1];
  useEffect(function () {
    var debouncedResizeHandler = lodash_debounce(function () {
      setDimensions({
        width: window.innerWidth || 0,
        height: window.innerHeight || 0
      });
    }, 50);
    window.addEventListener('resize', debouncedResizeHandler);
    return function () {
      return window.removeEventListener('resize', debouncedResizeHandler);
    };
  }, []);
  return {
    dimensions: dimensions
  };
};
var calculateChipsShownNumber = function (chipElements, containerElement, isExpanded) {
  var _a;
  if (!chipElements.length || !containerElement.current) return 0;
  var maxWidth = containerElement.current.offsetWidth;
  var additionalWidth = 8; /* chipElement left/right margin */
  var calculateReduceElements = function (start, skipAdditionalChip) {
    var _a;
    if (skipAdditionalChip === void 0) {
      skipAdditionalChip = false;
    }
    return chipElements.slice(start).reduce(function (_a, _b) {
      var items = _a.items,
        totalWidth = _a.totalWidth;
      var current = _b.current;
      if (!current) return {
        items: items,
        totalWidth: totalWidth
      };
      // approximate width for the chip number of hidden chips
      var chipNumberLength = !skipAdditionalChip ? (chipElements.length - items > 9 ? 40 : 32) + additionalWidth : 0;
      // last item doesn't need ChipNumber length
      var calculatedMaxWidth = items === chipElements.length - start ? maxWidth : maxWidth - chipNumberLength;
      // Width for one chip and any additional width(paddings)
      var calculatedChipWidth = totalWidth + current.offsetWidth + additionalWidth;
      return {
        items: calculatedMaxWidth > calculatedChipWidth ? items + 1 : items,
        totalWidth: calculatedChipWidth
      };
    }, {
      items: 1,
      totalWidth: chipElements[start - 1].current ? (((_a = chipElements[start - 1].current) === null || _a === void 0 ? void 0 : _a.offsetWidth) || 0) + additionalWidth : 0
    });
  };
  var firstRowValue = calculateReduceElements(1, isExpanded);
  var secondRowValue = {
    items: 0,
    totalWidth: 0
  };
  if (isExpanded && firstRowValue.totalWidth >= maxWidth && ((_a = chipElements[firstRowValue.items]) === null || _a === void 0 ? void 0 : _a.current)) {
    secondRowValue = calculateReduceElements(firstRowValue.items + 1);
  }
  return firstRowValue.items + secondRowValue.items;
};
var MultiInputTrigger = /*#__PURE__*/React.forwardRef(function (_a, ref) {
  var _b, _c, _d;
  var disabled = _a.disabled,
    label = _a.label,
    _e = _a.outline,
    outline = _e === void 0 ? 'default' : _e,
    _f = _a.autoFocus,
    autoFocus = _f === void 0 ? false : _f,
    triggerId = _a.triggerId,
    variant = _a.variant,
    _g = _a.visible,
    visible = _g === void 0 ? false : _g,
    _h = _a.onClick,
    onClick = _h === void 0 ? noop : _h,
    onClear = _a.onClear,
    onRemove = _a.onRemove,
    _j = _a.limitTwoRows,
    limitTwoRows = _j === void 0 ? true : _j,
    _k = _a.selectedOptions,
    selectedOptions = _k === void 0 ? [] : _k;
  var _l = useState([]),
    elRefs = _l[0],
    setElRefs = _l[1];
  var chipContainerRef = useRef(null);
  var numberChipRef = useRef(null);
  var _m = useState(visible),
    isVisibleInput = _m[0],
    setIsVisibleInput = _m[1];
  var _o = useState(limitTwoRows),
    showAllRows = _o[0],
    setShowAllRows = _o[1];
  var _p = useState(false),
    disableTabChip = _p[0],
    setDisableTabChip = _p[1];
  var dimensions = useResizeObserver().dimensions;
  var _q = useState(0),
    chipsShownNumber = _q[0],
    setChipsShownNumber = _q[1];
  useEffect(function () {
    setIsVisibleInput(visible);
    setShowAllRows(!limitTwoRows);
  }, [limitTwoRows, visible, disabled]);
  useEffect(function () {
    if (autoFocus && chipContainerRef.current) {
      chipContainerRef.current.focus();
    }
  }, [autoFocus]);
  useEffect(function () {
    setElRefs(function (prevState) {
      return Array(selectedOptions.length).fill(0).map(function (_, i) {
        return prevState[i] || /*#__PURE__*/createRef();
      });
    });
  }, [selectedOptions]);
  useEffect(function () {
    setChipsShownNumber(calculateChipsShownNumber(elRefs, chipContainerRef, isVisibleInput && !showAllRows));
  }, [isVisibleInput, elRefs, chipContainerRef, dimensions.width, showAllRows, disabled]);
  var expandInputTrigger = function () {
    setIsVisibleInput(true);
    setShowAllRows(true);
  };
  var hideChips = useCallback(function (index) {
    return (!showAllRows || !isVisibleInput) && index >= chipsShownNumber;
  }, [chipsShownNumber, isVisibleInput, showAllRows]);
  var showChipsNum = useCallback(function (index) {
    return (!showAllRows || !isVisibleInput) && index === chipsShownNumber - 1 && selectedOptions.length > chipsShownNumber;
  }, [selectedOptions.length, chipsShownNumber, isVisibleInput, showAllRows]);
  var focusChipElement = useCallback(function (index) {
    var _a;
    if (elRefs.length > 0 && elRefs[index].current) {
      (_a = elRefs[index].current) === null || _a === void 0 ? void 0 : _a.focus();
      setDisableTabChip(true);
    }
  }, [elRefs]);
  var onClickClearAll = useCallback(function (e) {
    e.stopPropagation();
    if (onClear) {
      onClear(e);
    }
    setTimeout(function () {
      var _a;
      (_a = chipContainerRef.current) === null || _a === void 0 ? void 0 : _a.focus();
    }, 0);
  }, [onClear]);
  var handleContainerKeyDown = useCallback(function (e) {
    if (['Enter', ' '].some(function (key) {
      return key === e.key;
    }) && e.target) {
      e.preventDefault();
      onClick();
    }
  }, [onClick]);
  var handleContainerFocus = useCallback(function (e) {
    if (!disableTabChip && e.target === e.currentTarget && elRefs.length > 0 && elRefs[0].current) {
      setDisableTabChip(true);
      elRefs[0].current.focus();
    }
  }, [disableTabChip, elRefs]);
  var handleContainerBlur = useCallback(function (e) {
    if (e.target !== e.currentTarget) {
      setDisableTabChip(false);
    }
  }, []);
  var handleChipNavigation = useCallback(function (index) {
    return function (event) {
      var _a, _b;
      switch (event.key) {
        case 'ArrowRight':
          if (index >= elRefs.length - 1) {
            focusChipElement(0);
            break;
          }
          if (selectedOptions.length !== chipsShownNumber && index + 1 >= chipsShownNumber) {
            if (!showAllRows) {
              (_a = numberChipRef.current) === null || _a === void 0 ? void 0 : _a.focus();
              setDisableTabChip(true);
              break;
            }
            expandInputTrigger();
          }
          setTimeout(function () {
            return focusChipElement(index + 1);
          }, 0);
          break;
        case 'ArrowLeft':
          if (index > 0) {
            focusChipElement(index - 1);
            break;
          }
          if (!showAllRows) {
            (_b = numberChipRef.current) === null || _b === void 0 ? void 0 : _b.focus();
            setDisableTabChip(true);
            break;
          }
          if (selectedOptions.length !== chipsShownNumber) {
            expandInputTrigger();
          }
          setTimeout(function () {
            return focusChipElement(elRefs.length - 1);
          }, 0);
          break;
      }
    };
  }, [chipsShownNumber, elRefs.length, focusChipElement, selectedOptions.length, showAllRows]);
  var handleChipRemoval = useCallback(function (index, option) {
    return function (e) {
      var _a, _b, _c, _d;
      if (!onRemove) return;
      e.stopPropagation();
      if (elRefs.length > 1) {
        (_c = ((_a = elRefs[index + 1]) === null || _a === void 0 ? void 0 : _a.current) || ((_b = elRefs[index - 1]) === null || _b === void 0 ? void 0 : _b.current)) === null || _c === void 0 ? void 0 : _c.focus();
        setDisableTabChip(true);
      } else {
        (_d = chipContainerRef.current) === null || _d === void 0 ? void 0 : _d.focus();
      }
      onRemove(option);
    };
  }, [elRefs, onRemove]);
  var handleNumberChipSelection = useCallback(function (e) {
    e.stopPropagation();
    expandInputTrigger();
  }, []);
  var handleNumberChipNavigation = useCallback(function (event) {
    switch (event.key) {
      case 'ArrowRight':
        focusChipElement(0);
        break;
      case 'ArrowLeft':
        focusChipElement(chipsShownNumber - 1);
        break;
      case 'Enter':
      case ' ':
        event.preventDefault();
        handleNumberChipSelection(event);
        setTimeout(function () {
          focusChipElement(chipsShownNumber);
        }, 0);
        break;
    }
  }, [chipsShownNumber, focusChipElement, handleNumberChipSelection]);
  return /*#__PURE__*/React.createElement(InputContainer, {
    containerRef: ref,
    outline: outline,
    onOutlineClick: onClick,
    className: classnames(styles$1.selectTrigger, (_b = {}, _b[styles$1.fullWidth] = variant === 'full-width', _b[styles$1.disabled] = disabled, _b)),
    inputClassName: classnames(styles$1.chipsContainer, (_c = {}, _c[styles$1.focused] = isVisibleInput, _c[styles$1.limitHeight] = !showAllRows, _c)),
    disabled: disabled,
    suffix: /*#__PURE__*/React.createElement("div", {
      className: styles$1.selectTriggerIcons
    }, onClear && selectedOptions.length > 0 ? /*#__PURE__*/React.createElement(IconButton, {
      icon: "times-circle",
      iconColor: !disabled ? 'icon-subdued' : 'icon-disabled',
      variant: "tertiary",
      "aria-label": "Clear Select Input",
      onClick: onClickClearAll,
      className: styles$1.icon,
      buttonState: disabled ? 'disabled' : 'enabled',
      tabIndex: 0
    }) : null, /*#__PURE__*/React.createElement(IconButton, {
      icon: visible ? 'angle-up' : 'angle-down',
      iconColor: !disabled ? 'icon-default' : 'icon-disabled',
      variant: "tertiary",
      tabIndex: -1,
      "aria-label": "Toggle visibility",
      className: styles$1.icon,
      buttonState: disabled ? 'disabled' : 'enabled'
    }))
  }, /*#__PURE__*/React.createElement("div", {
    ref: chipContainerRef,
    id: triggerId,
    role: "button",
    onKeyDown: handleContainerKeyDown,
    onFocus: handleContainerFocus,
    onBlur: handleContainerBlur,
    tabIndex: disableTabChip ? -1 : 0
  }, selectedOptions.map(function (option, index) {
    var _a, _b;
    return /*#__PURE__*/React.createElement(React.Fragment, {
      key: option.value
    }, /*#__PURE__*/React.createElement(SelectChip, {
      ref: elRefs[index],
      metadata: suffixMetadata(option.metadata, "tag-" + index),
      className: classnames((_a = {}, _a[styles$1.isHidden] = hideChips(index), _a)),
      onKeyDown: handleChipNavigation(index),
      onRemove: handleChipRemoval(index, option),
      disabled: disabled || (option === null || option === void 0 ? void 0 : option.disabled),
      tabIndex: -1
    }, option.label), showChipsNum(index) && /*#__PURE__*/React.createElement(SelectChip, {
      ref: numberChipRef,
      className: classnames((_b = {}, _b[styles$1.chipSelected] = isVisibleInput, _b)),
      onKeyDown: handleNumberChipNavigation,
      onClick: handleNumberChipSelection,
      disabled: disabled || (option === null || option === void 0 ? void 0 : option.disabled),
      tabIndex: -1
    }, "+" + (selectedOptions.length - chipsShownNumber)));
  }), /*#__PURE__*/React.createElement("input", {
    placeholder: !selectedOptions.length ? label : '',
    className: classnames(styles$1.inputElement, (_d = {}, _d[styles$1.hideContent] = selectedOptions.length > 0, _d)),
    size: (selectedOptions === null || selectedOptions === void 0 ? void 0 : selectedOptions.length) ? 1 : label === null || label === void 0 ? void 0 : label.length,
    disabled: disabled,
    tabIndex: -1,
    readOnly: true
  })));
});
var MIN_OPTIONS_TO_DISPLAY_SELECT_ALL = 7.5;
// Used for backward compatibility test check
function BackwardCompatibleSelectWithRef(_a, ref) {
  var className = _a.className,
    listClassName = _a.listClassName,
    _b = _a.closeOnSelect,
    closeOnSelect = _b === void 0 ? false : _b,
    _c = _a.disabled,
    disabled = _c === void 0 ? false : _c,
    _d = _a.variant,
    variant = _d === void 0 ? 'inline' : _d,
    _e = _a.selectAllLabel,
    selectAllLabel = _e === void 0 ? 'Select all' : _e,
    metadata = _a.metadata,
    _f = _a.multiple,
    multiple = _f === void 0 ? false : _f,
    _g = _a.onSearchChange,
    onSearchChange = _g === void 0 ? noop : _g,
    _h = _a.onSelect,
    onSelect = _h === void 0 ? noop : _h,
    _j = _a.onListVisibilityChange,
    onListVisibilityChange = _j === void 0 ? noop : _j,
    options = _a.options,
    _k = _a.emptyStateLabel,
    emptyStateLabel = _k === void 0 ? 'Nothing to display' : _k,
    renderEmptyState = _a.renderEmptyState,
    renderTrigger = _a.renderTrigger,
    _l = _a.optionType,
    optionType = _l === void 0 ? 'single-line' : _l,
    _m = _a.searchable,
    searchable = _m === void 0 ? true : _m,
    _o = _a.searchAutofocus,
    searchAutofocus = _o === void 0 ? true : _o,
    _p = _a.searchInitialValue,
    searchInitialValue = _p === void 0 ? '' : _p,
    _q = _a.searchNoResultsLabel,
    searchNoResultsLabel = _q === void 0 ? 'No results for' : _q,
    _r = _a.searchPlaceholder,
    searchPlaceholder = _r === void 0 ? 'Search' : _r,
    _s = _a.selectedOptions,
    selectedOptions = _s === void 0 ? [] : _s,
    _t = _a.minOptionsToDisplaySearch,
    minOptionsToDisplaySearch = _t === void 0 ? 8 : _t,
    _u = _a.triggerLabel,
    triggerLabel = _u === void 0 ? '' : _u,
    _v = _a.outline,
    outline = _v === void 0 ? 'default' : _v,
    _w = _a.autoFocus,
    autoFocus = _w === void 0 ? false : _w,
    triggerId = _a.triggerId,
    creatableOptions = _a.creatableOptions,
    _x = _a.isClearable,
    isClearable = _x === void 0 ? false : _x,
    _y = _a.highlightedText,
    highlightedText = _y === void 0 ? '' : _y,
    _z = _a.isLoading,
    isLoading = _z === void 0 ? false : _z,
    submitOptions = _a.submitOptions,
    _0 = _a.enableFlowTriggers,
    enableFlowTriggers = _0 === void 0 ? false : _0,
    _1 = _a.enableNewSearchInput,
    enableNewSearchInput = _1 === void 0 ? false : _1,
    _2 = _a.limitTwoRows,
    limitTwoRows = _2 === void 0 ? true : _2;
  var newTrigger = multiple ? /*#__PURE__*/React.createElement(MultiInputTrigger, {
    limitTwoRows: limitTwoRows
  }) : /*#__PURE__*/React.createElement(InputTrigger, null);
  var selectTrigger = !enableFlowTriggers ? renderTrigger || /*#__PURE__*/React.createElement(DefaultTrigger, null) : newTrigger;
  return /*#__PURE__*/React.createElement(BaseSelect, {
    ref: ref,
    className: className,
    variant: variant,
    metadata: metadata,
    searchInitialValue: searchInitialValue,
    selectedOptions: selectedOptions,
    minOptionsToDisplaySearch: minOptionsToDisplaySearch,
    isClearable: isClearable,
    isLoading: isLoading !== null && isLoading !== void 0 ? isLoading : false,
    onSearchChange: onSearchChange,
    onSelect: onSelect,
    onListVisibilityChange: onListVisibilityChange,
    multiple: multiple,
    optionType: optionType || 'single-line'
  }, /*#__PURE__*/React.createElement(BaseSelect.Trigger, {
    disabled: disabled || false,
    label: triggerLabel || '',
    outline: outline || 'default',
    autoFocus: autoFocus || false,
    triggerId: triggerId
  }, selectTrigger), /*#__PURE__*/React.createElement(BaseSelect.Content, {
    className: listClassName,
    metadata: metadata,
    closeOnSelect: closeOnSelect
  }, searchable && options.length >= minOptionsToDisplaySearch && /*#__PURE__*/React.createElement(BaseSelect.Content.SearchInput, {
    searchAutofocus: searchAutofocus !== null && searchAutofocus !== void 0 ? searchAutofocus : true,
    placeholder: searchPlaceholder || 'Search',
    enableNewSearchInput: enableNewSearchInput
  }), /*#__PURE__*/React.createElement(BaseSelect.Content.DefaultEmptyState, {
    label: emptyStateLabel || 'Nothing to display',
    searchNoResultsLabel: searchNoResultsLabel || 'No results for',
    customComponent: renderEmptyState
  }), /*#__PURE__*/React.createElement(BaseSelect.Content.ListBox, {
    searchable: searchable,
    highlightedText: highlightedText
  }, options.length > MIN_OPTIONS_TO_DISPLAY_SELECT_ALL && /*#__PURE__*/React.createElement(BaseSelect.Content.ListBox.SelectAllOption, {
    label: selectAllLabel
  }), generateOptionsAndOptionGroups(options)), !creatableOptions && submitOptions && /*#__PURE__*/React.createElement(BaseSelect.Content.SubmitFooter, {
    onSubmit: submitOptions.onSubmit,
    onCancel: submitOptions.onCancel,
    submitLabel: submitOptions.submitLabel,
    cancelLabel: submitOptions.cancelLabel
  }), creatableOptions && /*#__PURE__*/React.createElement(BaseSelect.Content.AddOptionsFooter, {
    onCreate: creatableOptions.onCreate,
    colorPickerOptions: creatableOptions.colorPickerOptions,
    multiple: multiple,
    createButtonProps: creatableOptions.createButtonProps,
    inputProps: creatableOptions.inputProps
  })));
}
BackwardCompatibleSelectWithRef.displayName = 'Select';
/**
 * According to this article: https://fettblog.eu/typescript-react-generic-forward-refs/
 */
var BackwardCompatibleSelect = /*#__PURE__*/React.forwardRef(BackwardCompatibleSelectWithRef);
export { BackwardCompatibleSelect as Select };
//# sourceMappingURL=index.js.map
